package org.artifactory.keys;

import lombok.experimental.UtilityClass;
import org.artifactory.api.rest.keys.KeyPairModel;
import org.artifactory.api.security.AuthorizationService;
import org.artifactory.common.crypto.CryptoHelper;
import org.artifactory.common.home.ArtifactoryHome;
import org.artifactory.descriptor.repo.KeyPairDescriptor;

/**
 * @author Yuval Reches
 */
@UtilityClass
public class KeyPairsUtil {

    public KeyPairDescriptor modelToDescriptor(KeyPairModel request) {
        KeyPairDescriptor model = new KeyPairDescriptor();
        model.setKey(request.getPairName());
        model.setKeyType(request.getPairType());
        model.setAlias(request.getAlias());
        model.setPrivateKey(request.getPrivateKey());
        model.setPublicKey(request.getPublicKey());
        model.setPassphrase(request.getPassphrase());
        return model;
    }

    public KeyPairModel toResponse(KeyPairDescriptor keyPairDescriptor, AuthorizationService authService) {
        return KeyPairModel.builder()
                .pairName(keyPairDescriptor.getKey())
                .pairType(keyPairDescriptor.getKeyType())
                .alias(keyPairDescriptor.getAlias())
                .publicKey(keyPairDescriptor.getPublicKey())
                .privateKey(authService.isAdmin() ? CryptoHelper.decryptIfNeeded(ArtifactoryHome.get(), keyPairDescriptor.getPrivateKey()) : null)
                .passphrase(authService.isAdmin() ? CryptoHelper.decryptIfNeeded(ArtifactoryHome.get(), keyPairDescriptor.getPassphrase()) : null)
                .build();
    }

}