package org.artifactory.opentracing;

/**
 * A runnable decorator that starts a server-side tracing span, either as an independent span or as a child of an existing
 * span
 *
 * @author Uriah Levy
 */
public class TraceableRunnableDecorator implements Runnable {
    private final ArtifactoryTracer artifactoryTracer;
    private Runnable runnable;
    private ServerTracingSpan currentSpan;

    public TraceableRunnableDecorator(Runnable runnable, ArtifactoryTracer artifactoryTracer) {
        this.artifactoryTracer = artifactoryTracer;
        this.currentSpan = artifactoryTracer.getCurrentThreadActiveSpan();
        this.runnable = runnable;
    }

    @Override
    public void run() {
        if (currentSpan != null) {
            // Set the current span as active for the current thread
            artifactoryTracer.setActiveSpan(currentSpan);
            runnable.run();
            return;
        }
        // Start a new span that ends at the end of this invocation
        try (ServerTracingScope ignored = artifactoryTracer.startSpanOnCurrentThread("Artifactory Async Invocation")) {
            runnable.run();
        }
    }
}
