package org.artifactory.version.converter.v300;

import org.artifactory.version.converter.XmlConverter;
import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.Namespace;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

import static org.artifactory.util.RepoLayoutUtils.GO_DEFAULT_NAME;

/**
 * @author Uriah Levy
 */
public class FixGoDefaultLayoutConverter implements XmlConverter {
    private static final Logger log = LoggerFactory.getLogger(FixGoDefaultLayoutConverter.class);
    private static final int NAME_INDEX = 1;
    private static final int ARTIFACT_PATH_PATTERN_INDEX = 3;
    private static final String FIXED_PATH_PATTERN = "[orgPath]/[module]/@v/v[baseRev](-[fileItegRev]).[ext]";

    @Override
    public void convert(Document doc) {
        log.info("Starting 'Go fix default repository layout' conversion");
        Element rootElement = doc.getRootElement();
        Namespace namespace = rootElement.getNamespace();

        Element repoLayoutsElement = rootElement.getChild("repoLayouts", namespace);
        List<Element> repoLayouts = repoLayoutsElement.getChildren();

        for (Element layout : repoLayouts) {
            if (layout.getContent() != null && layout.getContent().get(NAME_INDEX) != null
                    && GO_DEFAULT_NAME.equals(layout.getContent().get(NAME_INDEX).getValue())) {
                log.debug("Found go-default layout");
                layout.setContent(ARTIFACT_PATH_PATTERN_INDEX, getFixedArtifactPathPattern(namespace));
                log.debug("Fixed go-default layout successfully");
            }
        }
        log.info("Finished 'Go fix default repository layout' conversion successfully");
    }

    private Element getFixedArtifactPathPattern(Namespace namespace) {
        Element artifactPathPatternElement = new Element("artifactPathPattern", namespace);
        artifactPathPatternElement.setText(FIXED_PATH_PATTERN);
        return artifactPathPatternElement;
    }
}