:: Generates a server.xml under CATALINA_HOME/conf
:: This script expects,
::   CATALINA_HOME to be set
::   systemYamlHelper.bat to be present in current script's parent directory
@echo off & setlocal enabledelayedexpansion

set serverXml=%CATALINA_HOME%\conf\server.xml

call :resolveSystemKeys %serverXml%

@endlocal

:: End of script
goto :EOF

REM #######################################
REM # Resolve any system keys in a file to its respective values from system.yaml
REM # Syntax of key should be of the following format ${parent.child},
REM # A default value can be set by using || as field separator
REM # Example : ${artifactory.port}
REM #           ${node.ip||127.0.0.1}
REM # Globals:
REM #   YAML_VALUE
REM # Arguments:
REM #   target  - location to target file with keys (example /opt/jfrog/artifactory/server.xml)
REM # Returns:
REM #   None
REM #######################################
:resolveSystemKeys
    setlocal enabledelayedexpansion
    set target=%~1
    set keyList=
    set resolvedValue=
    set resolveDefaultValue=
    set emptyKeyWord=_EMPTY_
    set keyOrig=
    set displayValue=

    if not defined target goto resolveSystemKeysEnd
    if not exist %target% goto resolveSystemKeysEnd

    :: regex supports the following examples,
    ::   test.name
    ::   test_name
    ::   artifacts.[0].name
    ::   test.test.test
    ::   parent.child||sample_default_value
    set "keyRegex=[a-z]*[A-Z]*\:*[0-9]*[_\-]*\[*\]*\**\.*|*\/*"

    :: support upto 32 levels of special characters - test.test.test-test.test.test_test...test
    for /L %%a in (1,1,5) do (
        set "keyRegex=!keyRegex!!keyRegex!"
    )

    set "grepExec=%JF_PRODUCT_HOME%\app\third-party\grep\bin\grep.exe"

    for /f "usebackq delims= " %%f in (`type %target% ^| %grepExec% -o "\${!keyRegex!}"`) do (
        set keyOrig=%%f
        set resolveKey=%%f
        set resolveDefaultValue=

        REM :: convert ${key} => key
        set resolveKey=!resolveKey:}=!
        set resolveKey=!resolveKey:${=!

        REM Check if a defaut value is defined
        REM parent.child||sample_default
        REM resolveDefaultValue=sample_default
        REM resolveKey=parent.child
        echo "!resolveKey!" | findstr /C:"||" 1>nul 2>&1
        if !ERRORLEVEL! == 0 (
            for /f "tokens=1,2 delims=||" %%g in ("!resolveKey!") do (
                set resolveKey=%%g
                set resolveDefaultValue=%%h
            )
        )

        CALL %~dp0\systemYamlHelper.bat :getSystemValue !resolveKey! !resolveDefaultValue!
        set "resolvedValue=!YAML_VALUE!"

        if "!resolvedValue!" == "!emptyKeyWord!" set resolvedValue=
         
        call %~dp0\systemYamlHelper.bat :isKeySensitive "!resolveKey!"
        if "!isSensitive!" == "yes" ( set displayValue=__sensitive_key_hidden___ ) else ( set "displayValue=!resolvedValue!" )
        
        if "!resolvedValue!" == "!resolveDefaultValue!" (
            call %~dp0\artifactoryCommon.bat :log "resolved !resolveKey! to default value : !displayValue!"
        )

        set sedExec="%JF_PRODUCT_HOME%\app\third-party\sed\bin\sed.exe"

        REM Handle #backslash is considered as escape character, replace it with forward slash
        if defined resolvedValue set "resolvedValue=!resolvedValue:\=/!"
        REM Handle double quotes disappear after replacement with sed, replace it with single quote
        if defined resolvedValue set "resolvedValue=!resolvedValue:"='!"

        REM Sed does not remove temp files (sed*****) on usage of inplace update https://stackoverflow.com/a/2053064
        REM https://sourceforge.net/p/gnuwin32/bugs/500/
        REM This logic creates a temp file with replaced output and moves it to target
        set "sedTempFile=!target!.sed"

        !sedExec! -e "s,!keyOrig!,!resolvedValue!,g" "!target!" > "!sedTempFile!" && MOVE /Y "!sedTempFile!" "!target!" >nul 2>&1 || (
            set JF_ERROR_MSG="resolving !keyOrig! to !displayValue! failed, command used : !sedExec! -i -e ^"s,!keyOrig!,!displayValue!,g^" !target!"
            goto errorExit
        )

        if exist !sedTempFile! del /f !sedTempFile! >nul 2>&1
    )

    type %target% | %grepExec% -o "\${!keyRegex!}" >nul 2>&1

    if !ERRORLEVEL! == 0 (
        rem Support for nested variables
        call :resolveSystemKeys %target%
    )

exit /b 0

:errorExit
    echo.
    echo ERROR: %JF_ERROR_MSG%
    echo.
    @endlocal
exit /b 1

