:: This script starts migrating old configurations (db.properties files, default file, yaml files) to system.yaml

@ECHO off

SETLOCAL EnableDelayedExpansion

SET DEFAULT_ACCESS_PORT=8040
SET DEFAULT_RT_PORT=8081
SET AC_PORT_YAMLPATH=migration.xmlFiles.serverXml.access.port
SET AC_MAXTHREADS_YAMLPATH=migration.xmlFiles.serverXml.access.maxThreads
SET AC_EXTRACONFIG_YAMLPATH=migration.xmlFiles.serverXml.access.extraConfig
SET AC_SENDREASONPHRASE_YAMLPATH=migration.xmlFiles.serverXml.access.sendReasonPhrase
SET RT_PORT_YAMLPATH=migration.xmlFiles.serverXml.artifactory.port
SET RT_MAXTHREADS_YAMLPATH=migration.xmlFiles.serverXml.artifactory.maxThreads
SET RT_EXTRACONFIG_YAMLPATH=migration.xmlFiles.serverXml.artifactory.extraConfig
SET RT_SENDREASONPHRASE_YAMLPATH=migration.xmlFiles.serverXml.artifactory.sendReasonPhrase
SET RT_RELAXEDPATHCHARS_YAMLPATH=migration.xmlFiles.serverXml.artifactory.relaxedPathChars
SET RT_RELAXEDQUERYCHARS_YAMLPATH=migration.xmlFiles.serverXml.artifactory.relaxedQueryChars
SET ROUTER_PORT_YAMLPATH=migration.xmlFiles.serverXml.router.port
SET EXTRA_CONFIG_YAMLPATH=migration.xmlFiles.serverXml.extra.config
SET EXTRA_LISTENER_CONFIG_YAMLPATH=migration.xmlFiles.serverXml.extra.listener
SET RT_TOMCAT_HTTPSCONNECTOR_ENABLED=artifactory.tomcat.httpsConnector.enabled

CALL %~dp0\artifactoryEnv.bat

Rem Set path for third-party usage of sed, xmllint, awk
SET xmllint=%JF_PRODUCT_HOME%\app\third-party\libxml2\bin\xmllint.exe
SET awkExec=%JF_PRODUCT_HOME%\app\third-party\gawk\bin\awk.exe
SET sedExec=%JF_PRODUCT_HOME%\app\third-party\sed\bin\sed.exe

SET scriptName=%~n0%~x0
CALL :setMigrationLogFile || GOTO :EOF

Rem Call Main function
CALL :main
GOTO :EOF

:main
    CALL :initialize || GOTO :EOF
    CALL :migrationArtifactory || GOTO :EOF
    GOTO :EOF
EXIT /B 0

:initialize
    CALL :getDataDir || GOTO :EOF
    CALL :initHelper || GOTO :EOF
    CALL :checkMigrationInfoYaml || GOTO :EOF
    CALL :getProduct  || GOTO :EOF
EXIT /B 0

:migrationArtifactory
	CALL :checkArtifactoryVersion || GOTO :EOF
    CALL :bannerSection "STARTING MIGRATION"
	CALL :setSystemYamlPath
	CALL :checkDataDirExists
	CALL :migratePropertiesFiles || GOTO :EOF
	CALL :migrateYamlFile || GOTO :EOF
    CALL :migrateXmlFile  || GOTO :EOF
	CALL :commentNodeId   || GOTO :EOF
	CALL :bannerSection "MIGRATION COMPLETED SUCCESSFULLY"
EXIT /B 0

Rem Reference https://stackoverflow.com/questions/15807762/compare-version-numbers-in-batch-file
Rem Version Check FUNCTION STARTS
:compareVersions version1  version2
CALL :testVersions %~1 %~2
    IF !errorlevel! == 1 SET "errorMode=!errorlevel!"
    IF !errorlevel! == -1 SET "errorMode=!errorlevel!"
    IF !errorlevel! == 0 SET "errorMode=!errorlevel!"
EXIT /B 0

:testVersions  version1  version2
    SETLOCAL enableDelayedExpansion
    SET "v1=%~1"
    SET "v2=%~2"
    CALL :divideLetters v1
    CALL :divideLetters v2

:loop
    CALL :parseNode "!v1!" n1 v1
    CALL :parseNode "!v2!" n2 v2
    IF !n1! gtr !n2! EXIT /B 1
    IF !n1! lss !n2! EXIT /B -1
    IF not defined v1 IF not defined v2 EXIT /B 0
    IF not defined v1 EXIT /b -1
    IF not defined v2 EXIT /b 1
GOTO :loop

:parseNode  version  nodeVar  remainderVar
    FOR /f "tokens=1* delims=.,-" %%A in ("%~1") DO (
        SET "%~2=%%A"
        SET "%~3=%%B"
    )
EXIT /B 0

:divideLetters  versionVar
    FOR %%C in (a b c d e f g h i j k l m n o p q r s t u v w x y z) DO (
        SET "%~1=!%~1:%%C=.%%C!"
    )
EXIT /B 0
Rem Version Check FUNCTION Ends

Rem Compare product version with minProductVersion and maxProductVersion
:migrateCheckVersion
    SET productVersion=%~1
    SET minProductVersion=%~2
    SET maxProductVersion=%~3

    CALL :compareVersions !productVersion! !maxProductVersion! 
    IF !errorMode! EQU 0 SET "checkMax=1"
    IF !errorMode! EQU 1 SET "checkMax=1"
    IF !checkMax! EQU 1 (
        CALL :logger "Migration not necessary. !PRODUCT! is already !productVersion!"
        EXIT /B 1
        GOTO :EOF
    )

    CALL :compareVersions !productVersion! !minProductVersion!
    IF !errorMode! EQU 0 SET "checkMin=1"
    IF !errorMode! EQU 1 SET "checkMin=1"
    IF !checkMin! EQU 1 (
        CALL :logger "Detected !PRODUCT! !productVersion!, initiating migration"
    )

    CALL :compareVersions !productVersion! !minProductVersion!
    IF !errorMode! EQU -1 (
       CALL :logger "Current !PRODUCT! !productVersion! version is not supported for migration"
       EXIT /B 1
       GOTO :EOF
    )
EXIT /B 0

Rem Read properties from property file
:readKey
    SET property=%~1
    SET filePath=%~2

    FOR /f "eol=# tokens=1,* delims== " %%i in (!filePath!) DO (
        SET key=%%i
        SET value=%%j
        IF [!property!] == [!key!] (
           SET version=!value!
           EXIT /B 0
        ) 
    )
EXIT /B 0

Rem Validate Artifactory version
:getProductVersion
    SET minProductVersion=%~1
    SET maxProductVersion=%~2
    SET newfilePath=%~3
    SET property=%~4

    IF EXIST !newfilePath! (
        CALL :readKey "!property!" "!newfilePath!"
        SET productVersion=!version!
    ) ELSE (
        SET JF_ERROR_MSG=File [!newfilePath!] not found to get current version. Skipping migration
        CALL :logError !JF_ERROR_MSG!
        GOTO :EOF
    )
    IF [!productVersion!] == [] (
        SET JF_ERROR_MSG=No version found in file [!newfilePath!]. Skipping migration
        GOTO :logError !JF_ERROR_MSG!
        GOTO :EOF     
    )
    CALL :migrateCheckVersion !productVersion! !minProductVersion! !maxProductVersion! || GOTO :EOF
EXIT /B 0

Rem Set Artifactory product min version ,max version ,property and path to property file
:checkArtifactoryVersion
    SET minProductVersion=6.0.0
    SET maxProductVersion=7.0.0
    SET property=artifactory.version
    SET newfilePath=!NEW_DATA_DIR!\etc\artifactory\artifactory.properties
    SET oldfilePath=!OLD_DATA_DIR!\etc\artifactory.properties

    CALL :getProductVersion !minProductVersion! !maxProductVersion! !newfilePath! !property! || GOTO :EOF
EXIT /B 0

Rem Retrieve yamlValue from MIGRATION_SYSTEM_YAML
:retrieveYamlValue
   SET yamlPath=%~1
   SET value=%~2
   SET output=%~3
   SET message=%~4 

   IF [!yamlPath!] == [] ( 
        SET JF_ERROR_MSG=yamlPath is mandatory to get value from !MIGRATION_SYSTEM_YAML!
        GOTO :logError !JF_ERROR_MSG!
    )
   CALL %~dp0\systemYamlHelper.bat :getYamlValue !yamlPath! !MIGRATION_SYSTEM_YAML! false
   SET value=!YAML_VALUE!
   IF [!value!] == [] (
       IF !output! == Warning (
            SET JF_WARN_MSG=Empty value for !yamlPath! in [!MIGRATION_SYSTEM_YAML!]
            GOTO :logWarn !JF_WARN_MSG! 
       )
       IF !output! == Skip (
            GOTO :EOF  
       ) ELSE (
            SET JF_ERROR_MSG=!message!
            GOTO :logError  !JF_ERROR_MSG!
       )
    )
EXIT /B 0

Rem Set derby path and replace {db.home} in the property value [Url] 
:migrateResolveDerbyPath
    SET key=%~1
    SET value=%~2
    SET derbyPath=!NEW_DATA_DIR!\data\artifactory\derby
    SET derbyUrlValue=%value:{db.home}=!derbyPath!%
EXIT /B 0

:: Migrate properties values to system.yaml with new yamlPath
:propertyMigrate
    SET entry=%~1
    SET filepath=%~2
    SET fileName=%~3
    
    CALL :getLeftHandSide "!entry!"
    SET yamlPath=!firstEntry!
    IF [!yamlPath!] == [] (
          SET JF_WARN_MSG=yamlPath is empty for [!key!] in [!MIGRATION_SYSTEM_YAML!]
          GOTO :logWarn !JF_WARN_MSG!
          EXIT /B 0
    )
    CALL :getRightHandSide "!entry!"
    SET property=!secondEntry!
    IF [!property!] == [] (
          SET JF_WARN_MSG=Property is empty in map [!entry!] in the file [!MIGRATION_SYSTEM_YAML!]
          GOTO :logWarn !JF_WARN_MSG!
          EXIT /B 0
    )
    FOR /f "eol=# tokens=1,* delims== " %%i in (!NEW_DATA_DIR!\!filePath!\!fileName!) DO (
        SET key=%%i
        SET value=%%j
        IF [!property!] == [!key!] (
            IF [!key!] == [url] (
                CALL :migrateResolveDerbyPath "!key!" "!value!"
                SET value=!derbyUrlValue!
            )
            Rem Set yaml value for [shared.node.ip] by greping ip from the property [context.url] 
            IF [!key!] == [context.url] (
                FOR /F "tokens=*" %%g IN ('ECHO !value! ^| !awkExec! -F/ "{print $3}" ^| !sedExec! "s/:.*//"') DO ( SET ipValue=%%g)
                CALL %~dp0\systemYamlHelper.bat :setSystemValue "shared.node.ip" "!ipValue!"
                CALL :logger "Setting [shared.node.ip] with [!ipValue!] in system.yaml"
            )
            CALL %~dp0\systemYamlHelper.bat :setSystemValue "!yamlPath!" "!value!" 
            CALL :logger "Setting [!yamlPath!] with value of the property [!key!] in system.yaml"
            SET propertyOk=yes
            EXIT /B 0
        )
            SET propertyOK=no
    )
    IF [!propertyOk!] == [no] (
        CALL :logger "Property [!property!] not found in the file [!fileName!]"
    )
EXIT /B 0

Rem setHaEnabled if ha-node.properties is present
:setHaEnabled
    SETLOCAL EnableDelayedExpansion
    SET filePath=%~1
    IF EXIST !filePath! (
        CALL %~dp0\systemYamlHelper.bat :setSystemValue "shared.node.haEnabled" "true"
        CALL :logger "Setting [shared.node.haEnabled] with [true] in system.yaml"
    )
    ENDLOCAL
EXIT /B 0

Rem Retrieve info of properties files from MIGRATION_SYSTEM_YAML
:migratePropertiesFiles
    SETLOCAL EnableDelayedExpansion
    CALL :retrieveYamlValue "migration.propertyFiles.files" "fileList" "Skip"
    SET fileList=!YAML_VALUE!
    IF [!fileList!] == [] ( 
        EXIT /B 0
    )
    CALL :bannerSection "PROCESSING MIGRATION OF PROPERTY FILES"
    FOR %%b in (!fileList!) DO (
        CALL :bannerSubSection "Processing Migration of %%b"
        CALL :retrieveYamlValue "migration.propertyFiles.%%b.filePath" "filePath" "Warning"
        SET filePath=!YAML_VALUE!
        CALL :retrieveYamlValue "migration.propertyFiles.%%b.fileName" "fileName" "Warning"
        SET fileName=!YAML_VALUE!
        IF [!filePath!] == [] (
            EXIT /B 0
        )
        IF [!fileName!] == [] ( 
            EXIT /B 0
        ) 
        SET fileLocation=!NEW_DATA_DIR!\!filePath!\!fileName!
        IF EXIST !fileLocation! (
            CALL :logger "File [!fileName!] found in path [!NEW_DATA_DIR!\!filePath!]"
            CALL :setHaEnabled "!NEW_DATA_DIR!\!filePath!\ha-node.properties"
            CALL :retrieveYamlValue "migration.propertyFiles.%%b.map" "map" "Warning"
            SET map=!YAML_VALUE!
            IF [!map!] == [] ( 
                EXIT /B 0
            )
            FOR %%a in ("!map: =" "!") DO (
                SET entry=%%~a
                CALL :checkMapEntry "!entry!"
                IF !entryResult! == yes (
                    CALL :propertyMigrate "!entry!" "!filePath!" "!fileName!"
                ) ELSE (
                    SET JF_WARN_MSG=Map entry [!entry!] in [!MIGRATION_SYSTEM_YAML!] is not in correct format, correct format i.e yamlPath=property
                    CALL :logWarn !JF_WARN_MSG!
                )
            )
        ) ELSE (
        CALL :logger "File [!fileName!] was not found in path [!NEW_DATA_DIR!\!filePath!] to migrate"
        )
     )
    ENDLOCAL
EXIT /B 0

Rem Comment node.id in system.yaml, Add a commented Line above the node.id in system.yaml
:commentNodeId
    CALL :retrieveYamlValue "migration.propertyFiles.haNodeProperty.filePath" "filePath" "Skip"
    SET filePath=!YAML_VALUE!
    CALL :retrieveYamlValue "migration.propertyFiles.haNodeProperty.fileName" "fileName" "Skip"
    SET fileName=!YAML_VALUE!
    IF [!filePath!] == [] (
        EXIT /B 0
    )
    IF [!fileName!] == [] ( 
        EXIT /B 0
    )
    SET fileLocation=!NEW_DATA_DIR!\!filePath!\!fileName!
    SET grepExec="%JF_PRODUCT_HOME%\app\third-party\grep\bin\grep.exe"
    IF EXIST !fileLocation! (
       CALL %~dp0\systemYamlHelper.bat :getYamlValue "shared.node" "!SYSTEM_YAML_PATH!" "false"
       SET nodeValue=!YAML_VALUE!
       FOR /F "tokens=*" %%g IN ('ECHO "!nodeValue!" ^| !grepExec! -o "^id:"') DO ( SET idKey=%%g)
       SET regexString=!idKey!
       SET replaceString="#&"
       !sedExec! -i -e "s/!regexString!/!replaceString!/" !SYSTEM_YAML_PATH! || ( SET JF_WARN_MSG="Failed to replace the text in !file!" && GOTO :logWarn !JF_WARN_MSG!)
       SET text=# NOTE: node.id can be automatically determined based on the current hostname or be set using the SHARED_NODE_ID environment variable. There is no need to explicitly specify it here.
       !sedExec! -i -e "/!regexString!/i !text!" !SYSTEM_YAML_PATH! || ( SET JF_WARN_MSG="Failed to prepend the text in !file!" && GOTO :logWarn !JF_WARN_MSG!)
    )
EXIT /B 0

Rem Migrate value of key from yaml file to system.yaml with new yamlPath
:migrateYamlValue
    SETLOCAL EnableDelayedExpansion
    SET entry=%~1
    SET sourceFile=%~2
    SET fileName=%~3
    
    CALL :getLeftHandSide "!entry!"
    SET yamlPath=!firstEntry!
    IF [!yamlPath!] == [] (
        SET JF_WARN_MSG=yamlPath is empty for [!key!] in [!MIGRATION_SYSTEM_YAML!]
        CALL :logWarn !JF_WARN_MSG!
        EXIT /B 0
    )
    CALL :getRightHandSide "!entry!"
    SET key=!secondEntry!
    IF [!key!] == [] (
        SET JF_WARN_MSG=key is empty in map [!entry!] in the file [!MIGRATION_SYSTEM_YAML!]
        CALL :logWarn !JF_WARN_MSG!
        EXIT /B 0
    )
    CALL %~dp0\systemYamlHelper.bat :getYamlValue "!key!" "!sourceFile!" "false"
    SET value=!YAML_VALUE!
    Rem Update yaml value for [replicator.profiling.enabled] with true, if profilingDisabled is set to false and visa versa
    IF [!key!] == [profilingDisabled] (
        IF NOT [!value!] == [] (
            IF [!value!] == [false] (
                SET value=true
            ) ELSE (
                SET value=false
            )
        )
    )
    IF [!value!] == [] (
        CALL :logger "No value found for key [!key!] in [!fileName!]"
    ) ELSE ( 
     	CALL %~dp0\systemYamlHelper.bat :setSystemValue "!yamlPath!" "!value!"
        CALL :logger "Setting [!yamlPath!] with value of the key [!key!] in system.yaml"
    )  
    ENDLOCAL   
EXIT /B 0

Rem Retrieve info of yaml files from MIGRATION_SYSTEM_YAML
:migrateYamlFile
    SETLOCAL EnableDelayedExpansion
    CALL :retrieveYamlValue "migration.yaml.files" "files" "Skip"
    SET files=!YAML_VALUE!
    IF [!files!] == [] ( 
    EXIT /B 0
    )
    CALL :bannerSection "PROCESSING MIGRATION OF YAML FILES"
    FOR %%b in (!files!) DO (
        CALL :bannerSubSection "Processing Migration of %%b"
        CALL :retrieveYamlValue "migration.yaml.%%b.filePath" "filePath" "Warning"
        SET filePath=!YAML_VALUE!
        CALL :retrieveYamlValue "migration.yaml.%%b.fileName" "fileName" "Warning"
        SET fileName=!YAML_VALUE!
        IF [!filePath!] == [] (
        EXIT /B 0
        )
        IF [!fileName!] == [] ( 
        EXIT /B 0
        ) 
        SET fileLocation=!NEW_DATA_DIR!\!filePath!\!fileName!
        IF EXIST !fileLocation! (
            CALL :logger "File [!fileName!] found in path [!NEW_DATA_DIR!\!filePath!]"
            CALL :retrieveYamlValue "migration.yaml.%%b.map" "map" "Warning"
            SET map=!YAML_VALUE!
            IF [!map!] == [] ( 
            EXIT /B 0
            )
            FOR %%a in ("!map: =" "!") DO (
                SET entry=%%~a
                CALL :checkMapEntry "!entry!"
                IF !entryResult! == yes (
                    CALL :migrateYamlValue "!entry!" "!fileLocation!" "!fileName!"
                ) ELSE (
                    SET JF_WARN_MSG=Map entry [!entry!] in [!MIGRATION_SYSTEM_YAML!] is not in correct format, correct format i.e yamlPath=key
                    CALL :logWarn !JF_WARN_MSG!
                )
            )
        ) ELSE (
            CALL :logger "File [!fileName!] was not found in path [!NEW_DATA_DIR!\!filePath!] to migrate"
        )
    )
    ENDLOCAL
EXIT /B 0

Rem Get attribute value of connector
:getConnectorAttributeValue
    SET i=%~1
    SET filePath=%~2
    SET fileName=%~3
    SET attribute=%~4
    FOR /F "tokens=*" %%g IN ('^"!xmllint! --xpath "//Server/Service/Connector[!i!]/@!attribute!" "!filePath!\!fileName!" 2^>nul ^| !awkExec! -F"=" "{print $2}"^"') DO ( SET attributeValue=%%~g)
EXIT /B 0

Rem Get all attribute value of connector
:getConnectorAttributes
    SET i=%~1
    SET filePath=%~2
    SET fileName=%~3
    SET connectorAttributes=
    FOR /F "tokens=*" %%g IN ('^"!xmllint! --xpath "//Server/Service/Connector[!i!]/@*" "!filePath!\!fileName!" 2^>nul^"') DO ( SET connectorAttributes=%%g)
EXIT /B 0

Rem Updating system.yaml with connector attribute value
:setConnectorAttributeValue
    SETLOCAL EnableDelayedExpansion
    SET yamlPath=%~1
    IF [!yamlPath!] == [] (
        EXIT /B 0
    )
    SET attribute=%~2
    SET attributeValue=%~3
    IF [!attributeValue!] == [] (
        EXIT /B 0
    )
    Rem Getting !attribute! yaml path from migration info yaml
    CALL :retrieveYamlValue "!yamlPath!" "attributeYamlPath" "Warning"
    SET attributeYamlPath=!YAML_VALUE!
    IF [!attributeYamlPath!] == [] (
        EXIT /B 0
    )
    CALL %~dp0\systemYamlHelper.bat :setSystemValue "!attributeYamlPath!" "!attributeValue!"
    CALL :logger "Setting [!attributeYamlPath!] with value [!attributeValue!] in system.yaml"
    ENDLOCAL
EXIT /B 0

Rem Updating system.yaml with connector attributes
:setConnectorAttributes
    SETLOCAL EnableDelayedExpansion
    SET yamlPath=%~1
    IF [!yamlPath!] == [] (
        EXIT /B 0
    )
    SET attribute=%2
    SET attributeValue=%~3
    IF [!attributeValue!] == [] (
        EXIT /B 0
    )
    Rem Getting !attribute! yaml path from migration info yaml
    CALL :retrieveYamlValue "!yamlPath!" "attributeYamlPath" "Warning"
    SET attributeYamlPath=!YAML_VALUE!
    IF [!attributeYamlPath!] == [] (
        EXIT /B 0
    )
    Rem Replace (') with (")
    SET attributeValue=!attributeValue:'="!
    Rem Replace (") with (""")
    SET attributeValue=!attributeValue:"="""!
    CALL %~dp0\systemYamlHelper.bat :setSystemValue "!attributeYamlPath!" ^"!attributeValue!^"
    CALL :logger "Setting [!attributeYamlPath!] with connector attributes in system.yaml"
    ENDLOCAL
EXIT /B 0

Rem Updating system.yaml with extra connector
:setExtraConnector
    SETLOCAL EnableDelayedExpansion
    SET yamlPath=%~1
    IF [!yamlPath!] == [] (
        EXIT /B 0
    )
    SET attributeValue=%~2
    IF [!attributeValue!] == [] (
        EXIT /B 0
    )

    Rem Getting !attribute! yaml path from migration info yaml
    CALL :retrieveYamlValue "!yamlPath!" "attributeYamlPath" "Warning"
    SET attributeYamlPath=!YAML_VALUE!
    IF [!attributeYamlPath!] == [] (
        EXIT /B 0
    )

    CALL %~dp0\systemYamlHelper.bat :getYamlValue "!attributeYamlPath!" !SYSTEM_YAML_PATH! false
    SET connectorExtra=!YAML_VALUE!
    IF [!connectorExtra!] == [] (
        CALL %~dp0\systemYamlHelper.bat :setSystemValue "!attributeYamlPath!" ^"!attributeValue!^"
        CALL :logger "Setting [!attributeYamlPath!] with extra connectors in system.yaml"
    ) ELSE (
        SET connectorExtra=!connectorExtra:'="!
        SET connectorExtra=!connectorExtra:"="""!
        SET connectorExtra=!connectorExtra:^<=@gt@!
        SET connectorExtra=!connectorExtra:^>=@lt@!
        CALL %~dp0\systemYamlHelper.bat :setSystemValue "!attributeYamlPath!" ^"!connectorExtra! ^^^ !attributeValue!^"
        CALL :logger "Setting [!attributeYamlPath!] with extra connectors in system.yaml"
    )
    ENDLOCAL
EXIT /B 0

Rem Migrate extra connectors to system.yaml
:migrateExtraConnectors
    SETLOCAL EnableDelayedExpansion
    SET filePath=%~1
    SET fileName=%~2
    SET connectorCount=%~3
    SET excludeDefaultPort=%~4
    SET i=%~5
    IF [!excludeDefaultPort!] == [yes] (
        FOR /L %%g IN (1,1,!connectorCount!) DO (
            CALL :getConnectorAttributeValue %%g !filePath! !fileName! "port"
            IF NOT [!attributeValue!] == [!DEFAULT_RT_PORT!] (
                IF NOT [!attributeValue!] == [!DEFAULT_ACCESS_PORT!] ( 
                    SET j=%%g
                    FOR /F "tokens=*" %%f IN ('^"!xmllint! --xpath "//Server/Service/Connector[!j!]" "!filePath!\!fileName!" 2^>nul^"') DO ( SET extraConnector=%%f)
                    SET extraConnector=!extraConnector:'="!
                    SET extraConnector=!extraConnector:"="""!
                    SET extraConnector=!extraConnector:^<=@gt@!
                    SET extraConnector=!extraConnector:^>=@lt@!
                    CALL :setExtraConnector "!EXTRA_CONFIG_YAMLPATH!" "!extraConnector!"
                )
            )
        )
    ) ELSE (
        FOR /F "tokens=*" %%g IN ('^"!xmllint! --xpath "//Server/Service/Connector[!i!]" "!filePath!\!fileName!" 2^>nul^"') DO ( SET extraConnector=%%g)
        SET extraConnector=!extraConnector:'="!
        SET extraConnector=!extraConnector:"="""!
        SET extraConnector=!extraConnector:^<=@gt@!
        SET extraConnector=!extraConnector:^>=@lt@!
        CALL :setExtraConnector "!EXTRA_CONFIG_YAMLPATH!" "!extraConnector!"
    )
    ENDLOCAL 
EXIT /B 0

Rem Migrate connector configurations
:migrateConnectorConfig
    SETLOCAL EnableDelayedExpansion
    SET filePath=%~1
    SET fileName=%~2
    SET i=%~3
    SET portValue=%~4
    SET portYamlPath=%~5
    SET maxThreadYamlPath=%~6
    SET extraConfigYamlPath=%~7
    SET sendReasonPhraseYamlPath=%~8
    SET relaxedPathCharsYamlPath=%~9
    SHIFT
    SET relaxedQueryCharsYamlPath=%~9

    Rem migrate port
    CALL :setConnectorAttributeValue "!portYamlPath!" "port" "!portValue!"
    Rem migrate maxThread ,sendReasonPhrase , relaxedPathChars , relaxedQueryChars
    SET Arr[0]=maxThreads
    SET Arr[1]=sendReasonPhrase
    SET Arr[2]=relaxedPathChars
    SET Arr[3]=relaxedQueryChars
    FOR /F "tokens=2 delims==" %%s IN ('set Arr[') DO (
        CALL :getConnectorAttributeValue "!i!" "!filePath!" "!fileName!" %%s
        IF [%%s] == [maxThreads] ( 
            SET connectorYamlPath=!maxThreadYamlPath!
            SET maxthreadValue=!attributeValue!
            CALL :setConnectorAttributeValue "!connectorYamlPath!" %%s "!maxthreadValue!"
        )
        IF [%%s] == [sendReasonPhrase] ( 
            SET connectorYamlPath=!sendReasonPhraseYamlPath!
            SET sendReasonPhraseValue=!attributeValue!
            CALL :setConnectorAttributeValue "!connectorYamlPath!" %%s "!sendReasonPhraseValue!"
        )
        IF [%%s] == [relaxedPathChars] ( 
            SET connectorYamlPath=!relaxedPathCharsYamlPath!
            SET relaxedPathCharsValue=""""!attributeValue!""""
            CALL :setConnectorAttributeValue "!connectorYamlPath!" %%s !relaxedPathCharsValue!
        )
        IF [%%s] == [relaxedQueryChars] ( 
            SET connectorYamlPath=!relaxedQueryCharsYamlPath!
            SET relaxedQueryCharsValue=""""!attributeValue!""""
            CALL :setConnectorAttributeValue "!connectorYamlPath!" %%s !relaxedQueryCharsValue!
        ) 
    )

    Rem migrate extra attributes of connector
    CALL :getConnectorAttributes "!i!" "!filePath!" "!fileName!"
    FOR /F "tokens=*" %%g IN ('ECHO !connectorAttributes! ^| !sedExec! "s/port=\"%portValue%\"//g"') DO ( SET connectorAttributes=%%~g)
    FOR /F "tokens=*" %%g IN ('ECHO !connectorAttributes! ^| !sedExec! "s/maxThreads=\"%maxthreadValue%\"//g"') DO ( SET connectorAttributes=%%~g)
    FOR /F "tokens=*" %%g IN ('ECHO !connectorAttributes! ^| !sedExec! "s/sendReasonPhrase=\"%sendReasonPhraseValue%\"//g"') DO ( SET connectorAttributes=%%~g)
    IF NOT [!connectorAttributes!] == [] (
        FOR /F "tokens=*" %%g IN ('ECHO !connectorAttributes! ^| !sedExec! "s/relaxedPathChars=\"\[\]\"//g"') DO ( SET connectorAttributes=%%~g)
        FOR /F "tokens=*" %%g IN ('ECHO !connectorAttributes! ^| !sedExec! "s/relaxedQueryChars=\"\[\]\"//g"') DO ( SET connectorAttributes=%%~g)
    )
    IF NOT [!connectorAttributes!] == [] (
        CALL :trim connectorAttributes %connectorAttributes%
        CALL :setConnectorAttributes "!extraConfigYamlPath!" allExtraAttributes "!connectorAttributes!"
    )
    ENDLOCAL
EXIT /B 0

Rem remove trailing spaces in variables
:trim
    SETLOCAL EnableDelayedExpansion
    SET Params=%*
    FOR /F "tokens=1*" %%a in ("!Params!") DO ENDLOCAL & SET %1=%%b
EXIT /B 0

Rem Check for default port 8040 and 8081 in connectors and migrate
:migrateConnectorPort
    SET filePath=%~1
    SET fileName=%~2
    SET connectorCount=%~3
    SET defaultPort=%~4
    SET portYamlPath=%~5
    SET maxThreadYamlPath=%~6
    SET extraConfigYamlPath=%~7
    SET sendReasonPhraseYamlPath=%~8
    SET relaxedPathCharsYamlPath=%~9
    SHIFT
    SET relaxedQueryCharsYamlPath=%~9
    
    FOR /L %%g IN (1,1,!connectorCount!) DO (
        CALL :getConnectorAttributeValue %%g "!filePath!" "!fileName!" "port"
        SET portValue=!attributeValue!
        CALL :getConnectorAttributeValue %%g "!filePath!" "!fileName!" "protocol"
        SET protocolType=!attributeValue!
        IF /I "!protocolType:AJP=!" NEQ "!protocolType!" (  
            SET AJP=yes
        )  ELSE (
            SET AJP=no
            IF [!portValue!] == [!defaultPort!] (  
                IF [!portValue!] == [!DEFAULT_RT_PORT!] (
                    SET RT_DEFAULTPORT_STATUS=yes
                ) ELSE (
                    SET AC_DEFAULTPORT_STATUS=yes
                )
                CALL :migrateConnectorConfig "!filePath!" "!fileName!" %%g "!portValue!" "!portYamlPath!" "!maxThreadYamlPath!" "!extraConfigYamlPath!" "!sendReasonPhraseYamlPath!" "!relaxedPathCharsYamlPath!" "!relaxedQueryCharsYamlPath!"
            )
        )
    )
EXIT /B 0

Rem migrate to extra, connector having default port and protocol is AJP
:migrateDefaultPortIfAjp
    SETLOCAL EnableDelayedExpansion
    SET filePath=%~1
    SET fileName=%~2
    SET connectorCount=%~3
    SET defaultPort=%~4
    FOR /L %%g IN (1,1,!connectorCount!) DO (
        CALL :getConnectorAttributeValue %%g "!filePath!" "!fileName!" "port"
        SET portValue=!attributeValue!
        CALL :getConnectorAttributeValue %%g "!filePath!" "!fileName!" "protocol"
        SET protocolType=!attributeValue!
        IF /I "!protocolType:AJP=!" NEQ "!protocolType!" (
            SET AJP=yes
            IF [!portValue!] == [!defaultPort!] (
                CALL :migrateExtraConnectors "!filePath!" "!fileName!" "!connectorCount!" "no" %%g 
            )
        )  ELSE (
            SET AJP=false
        )
    )
    ENDLOCAL
EXIT /B 0

Rem Comparing max threads in connectors
:compareMaxThreads
    SETLOCAL EnableDelayedExpansion
    SET /a firstConnectorMaxThread=%~1
    SET firstConnectorNode=%~2
    SET /a secondConnectorMaxThread=%~3
    SET secondConnectorNode=%~4
    SET filePath=%~5
    SET fileName=%~6

    Rem choose higher maxThreads connector as Artifactory.
    SET compareResult=no
    IF !firstConnectorMaxThread! GTR !secondConnectorMaxThread! SET compareResult=yes
    IF !firstConnectorMaxThread! EQU !secondConnectorMaxThread! SET compareResult=yes
    IF [!compareResult!] == [yes] (
        Rem maxThread is higher in firstConnector,
        Rem Taking firstConnector as Artifactory and SecondConnector as Access
        Rem maxThread is equal in both connector,considering firstConnector as Artifactory and SecondConnector as Access
        CALL :getConnectorAttributeValue "!firstConnectorNode!" "!filePath!" "!fileName!" "port"
        SET rtPortValue=!attributeValue!
        CALL :migrateConnectorConfig "!filePath!" "!fileName!" !firstConnectorNode! "!rtPortValue!" "!RT_PORT_YAMLPATH!" "!RT_MAXTHREADS_YAMLPATH!" "!RT_EXTRACONFIG_YAMLPATH!" "!RT_SENDREASONPHRASE_YAMLPATH!" "!RT_RELAXEDPATHCHARS_YAMLPATH!" "!RT_RELAXEDQUERYCHARS_YAMLPATH!"
        CALL :getConnectorAttributeValue "!secondConnectorNode!" "!filePath!" "!fileName!" "port"
        SET acPortValue=!attributeValue!
        CALL :migrateConnectorConfig "!filePath!" "!fileName!" "!secondConnectorNode!" "!acPortValue!" "!AC_PORT_YAMLPATH!" "!AC_MAXTHREADS_YAMLPATH!" "!AC_EXTRACONFIG_YAMLPATH!" "!AC_SENDREASONPHRASE_YAMLPATH!" "!AC_RELAXEDPATHCHARS_YAMLPATH!" "!AC_RELAXEDQUERYCHARS_YAMLPATH!"
    ) ELSE (
        Rem maxThread is higher in SecondConnector, 
        Rem Taking SecondConnector as Artifactory and firstConnector as Access
        CALL :getConnectorAttributeValue "!secondConnectorNode!" "!filePath!" "!fileName!" "port"
        SET rtPortValue=!attributeValue!
        CALL :migrateConnectorConfig "!filePath!" "!fileName!" !secondConnectorNode! "!rtPortValue!" "!RT_PORT_YAMLPATH!" "!RT_MAXTHREADS_YAMLPATH!" "!RT_EXTRACONFIG_YAMLPATH!" "!RT_SENDREASONPHRASE_YAMLPATH!" "!RT_RELAXEDPATHCHARS_YAMLPATH!" "!RT_RELAXEDQUERYCHARS_YAMLPATH!"
        CALL :getConnectorAttributeValue "!firstConnectorNode!" "!filePath!" "!fileName!" "port"
        SET acPortValue=!attributeValue!
        CALL :migrateConnectorConfig "!filePath!" "!fileName!" "!firstConnectorNode!" "!acPortValue!" "!AC_PORT_YAMLPATH!" "!AC_MAXTHREADS_YAMLPATH!" "!AC_EXTRACONFIG_YAMLPATH!" "!AC_SENDREASONPHRASE_YAMLPATH!" "!AC_RELAXEDPATHCHARS_YAMLPATH!" "!AC_RELAXEDQUERYCHARS_YAMLPATH!"
    )
    ENDLOCAL
EXIT /B 0

Rem Check max threads exist to compare
:maxThreadsExistToCompare
    SETLOCAL EnableDelayedExpansion
    SET filePath=%~1
    SET fileName=%~2
    SET connectorCount=%~3
    SET maxThreadValueEmpty=yes
    SET firstnode=no

    Rem migrate extra connector only if connector is AJP
    FOR /L %%g IN (1,1,!connectorCount!) DO (
        SET protocolType=
        CALL :getConnectorAttributeValue %%g "!filePath!" "!fileName!" "protocol"
        SET protocolType=!attributeValue!
        IF [!protocolType!] == [] ( 
            SET AJP=false
        ) ELSE (
            IF "!protocolType:AJP=!" NEQ "!protocolType!" (
                SET AJP=yes
                CALL :migrateExtraConnectors "!filePath!" "!fileName!" "!connectorCount!" "no" %%g
            ) ELSE (
                SET AJP=no
            )
        )
    )

    Rem set nonAJP connector for first and second
    FOR /L %%g IN (1,1,!connectorCount!) DO (
        SET protocolType=
        CALL :getConnectorAttributeValue %%g "!filePath!" "!fileName!" "protocol"
        SET protocolType=!attributeValue!
        IF [!protocolType!] == [] (
            SET AJP=false
            IF [!firstnode!] == [no] (
                CALL :getConnectorAttributeValue %%g "!filePath!" "!fileName!" "maxThreads"
                SET /a firstConnectorMaxThread=!attributeValue!
                SET firstConnectorNode=%%g
                SET firstnode=yes
            ) ELSE (
                CALL :getConnectorAttributeValue %%g "!filePath!" "!fileName!" "maxThreads"
                SET /a secondConnectorMaxThread=!attributeValue!
                SET secondConnectorNode=%%g
            )
        ) ELSE ( 
            IF /I "!protocolType:AJP=!" NEQ "!protocolType!" (
                SET AJP=yes
            ) ELSE (
                SET AJP=no
                IF [!firstnode!] == [no] (
                    CALL :getConnectorAttributeValue %%g "!filePath!" "!fileName!" "maxThreads"
                    SET firstConnectorMaxThread=!attributeValue!
                    SET firstConnectorNode=%%g
                    SET firstnode=yes
                ) ELSE (
                    CALL :getConnectorAttributeValue %%g "!filePath!" "!fileName!" "maxThreads"
                    SET secondConnectorMaxThread=!attributeValue!
                    SET secondConnectorNode=%%g
                )
            )
        )
    )

    IF  [!firstConnectorMaxThread!] == [] ( SET maxThreadValueEmpty=no )
    IF  [!secondConnectorMaxThread!] == [] ( SET maxThreadValueEmpty=no )
    Rem maxThreads is set, now compare MaxThreads
    IF [!maxThreadValueEmpty!] == [yes] (
        CALL :compareMaxThreads "!firstConnectorMaxThread!" "!firstConnectorNode!" "!secondConnectorMaxThread!" "!secondConnectorNode!" "!filePath!" "!fileName!"
    ) ELSE (
        Rem Assume first connector is RT, maxThreads is not set in both connectors
        CALL :getConnectorAttributeValue "!firstConnectorNode!" "!filePath!" "!fileName!" "port"
        SET rtPortValue=!attributeValue!
        CALL :migrateConnectorConfig "!filePath!" "!fileName!" !firstConnectorNode! "!rtPortValue!" "!RT_PORT_YAMLPATH!" "!RT_MAXTHREADS_YAMLPATH!" "!RT_EXTRACONFIG_YAMLPATH!" "!RT_SENDREASONPHRASE_YAMLPATH!" "!RT_RELAXEDPATHCHARS_YAMLPATH!" "!RT_RELAXEDQUERYCHARS_YAMLPATH!"
        CALL :getConnectorAttributeValue "!secondConnectorNode!" "!filePath!" "!fileName!" "port"
        SET acPortValue=!attributeValue!
        CALL :migrateConnectorConfig "!filePath!" "!fileName!" "!secondConnectorNode!" "!acPortValue!" "!AC_PORT_YAMLPATH!" "!AC_MAXTHREADS_YAMLPATH!" "!AC_EXTRACONFIG_YAMLPATH!" "!AC_SENDREASONPHRASE_YAMLPATH!" "!AC_RELAXEDPATHCHARS_YAMLPATH!" "!AC_RELAXEDQUERYCHARS_YAMLPATH!"
    )
    ENDLOCAL
EXIT /B 0

Rem find RT and AC Connector
:findRtAndAcConnector
    SETLOCAL EnableDelayedExpansion
    SET filePath=%~1
    SET fileName=%~2
    SET connectorCount=%~3
    SET initialAjpCount=0
    SET nonAjpCount=0

    Rem get the count of non AJP
    FOR /L %%g IN (1,1,!connectorCount!) DO (
        SET protocolType=
        CALL :getConnectorAttributeValue %%g "!filePath!" "!fileName!" "protocol"
        SET protocolType=!attributeValue!
        IF [!protocolType!] == [] ( 
            SET /a nonAjpCount=!initialNonAjpCount!+1
            SET initialNonAjpCount=!nonAjpCount!
        ) ELSE (
            IF "!protocolType:AJP=!" NEQ "!protocolType!" (
                SET AJP=yes
            ) ELSE (
                SET AJP=no
                SET /a nonAjpCount=!initialNonAjpCount!+1
                SET initialNonAjpCount=!nonAjpCount!
            )
        )
    )
    IF !nonAjpCount! EQU 1 (
        Rem Add the connector found as access and artifactory connectors
        Rem Mark port as 8040 for access
        FOR /L %%g IN (1,1,!connectorCount!) DO (
            SETLOCAL
            CALL :getConnectorAttributeValue %%g "!filePath!" "!fileName!" "protocol"
            SET protocolType=!attributeValue!
            IF /I "!protocolType:AJP=!" NEQ "!protocolType!" (
                SET AJP=yes
                CALL :migrateExtraConnectors "!filePath!" "!fileName!" "!connectorCount!" "no" %%g
            ) ELSE (
                SET AJP=no
                CALL :getConnectorAttributeValue %%g "!filePath!" "!fileName!" "port"
                SET portValue=!attributeValue!
                CALL :migrateConnectorConfig "!filePath!" "!fileName!" %%g "!portValue!" "!RT_PORT_YAMLPATH!" "!RT_MAXTHREADS_YAMLPATH!" "!RT_EXTRACONFIG_YAMLPATH!" "!RT_SENDREASONPHRASE_YAMLPATH!" "!RT_RELAXEDPATHCHARS_YAMLPATH!" "!RT_RELAXEDQUERYCHARS_YAMLPATH!" 
                CALL :migrateConnectorConfig "!filePath!" "!fileName!" %%g "!portValue!" "!AC_PORT_YAMLPATH!" "!AC_MAXTHREADS_YAMLPATH!" "!AC_EXTRACONFIG_YAMLPATH!" "!AC_SENDREASONPHRASE_YAMLPATH!" "!AC_RELAXEDPATHCHARS_YAMLPATH!" "!AC_RELAXEDQUERYCHARS_YAMLPATH!"
                CALL :setConnectorAttributeValue "!AC_PORT_YAMLPATH!" "port" "!DEFAULT_ACCESS_PORT!"
            )
            ENDLOCAL
        )
    )
    IF !nonAjpCount! EQU 2 (
        Rem compare maxThreads in both connectors
        CALL :maxThreadsExistToCompare "!filePath!" "!fileName!" "!connectorCount!"
    )
    IF !nonAjpCount! GTR  2 (
        Rem migrateExtraConnectors
        CALL :migrateExtraConnectors "!filePath!" "!fileName!" "!connectorCount!" "yes"
    )
    IF !nonAjpCount! EQU 0 (
        Rem setting with default port in system.yaml
        CALL :setConnectorAttributeValue "!RT_PORT_YAMLPATH!" "port" "!DEFAULT_RT_PORT!"
        CALL :setConnectorAttributeValue "!AC_PORT_YAMLPATH!" "port" "!DEFAULT_ACCESS_PORT!"
        Rem migrateExtraConnectors
        CALL :migrateExtraConnectors "!filePath!" "!fileName!" "!connectorCount!" "yes"
    )
    ENDLOCAL
EXIT /B 0

Rem get the count of non AJP
:getCountOfNonAjp
    SET port=%~1
    SET connectorCount=%~2
    SET filePath=%~3
    SET fileName=%~4
    SET initialNonAjpCount=0

    FOR /L %%g IN (1,1,!connectorCount!) DO (
        SET protocolType=
        CALL :getConnectorAttributeValue %%g "!filePath!" "!fileName!" "port"
        SET portValue=!attributeValue!
        IF NOT [!portValue!] == [!port!] (
            CALL :getConnectorAttributeValue %%g "!filePath!" "!fileName!" "protocol"
            SET protocolType=!attributeValue!
            IF [!protocolType!] == [] ( 
                SET /a nonAjpCount=!initialNonAjpCount!+1
                SET initialNonAjpCount=!nonAjpCount!
            ) ELSE (
                IF "!protocolType:AJP=!" NEQ "!protocolType!" (
                    SET AJP=yes
                ) ELSE (
                    SET AJP=no
                    SET /a nonAjpCount=!initialNonAjpCount!+1
                    SET initialNonAjpCount=!nonAjpCount!
                )
            )
        )
    )
EXIT /B 0

Rem Find for Access connector
:findAcConnector
    SETLOCAL EnableDelayedExpansion
    SET filePath=%~1
    SET fileName=%~2
    SET connectorCount=%~3
    SET nonAjpCount=0

    Rem get the count of non AJP 
    CALL :getCountOfNonAjp "!DEFAULT_RT_PORT!" "!connectorCount!" "!filePath!" "!fileName!"
    IF !nonAjpCount! EQU 1 (
        Rem Add the connector found as access connector and mark port as that of connector
        FOR /L %%g IN (1,1,!connectorCount!) DO (
            SETLOCAL
            CALL :getConnectorAttributeValue %%g "!filePath!" "!fileName!" "port"
            SET portValue=!attributeValue!
            CALL :getConnectorAttributeValue %%g "!filePath!" "!fileName!" "protocol"
            SET protocolType=!attributeValue!
            IF NOT [!portValue!] == [!DEFAULT_RT_PORT!] (
                IF /I "!protocolType:AJP=!" NEQ "!protocolType!" (
                    SET AJP=yes
                    CALL :migrateExtraConnectors "!filePath!" "!fileName!" "!connectorCount!" "no" %%g
                ) ELSE (
                    SET AJP=no
                    CALL :migrateConnectorConfig "!filePath!" "!fileName!" %%g "!portValue!" "!AC_PORT_YAMLPATH!" "!AC_MAXTHREADS_YAMLPATH!" "!AC_EXTRACONFIG_YAMLPATH!" "!AC_SENDREASONPHRASE_YAMLPATH!" "!AC_RELAXEDPATHCHARS_YAMLPATH!" "!AC_RELAXEDQUERYCHARS_YAMLPATH!"
                )  
            )
            ENDLOCAL
        )
    )
    IF !nonAjpCount! GTR 1 (
        Rem Take RT properties into access with 8040
        FOR /L %%g IN (1,1,!connectorCount!) DO (
            SETLOCAL
            CALL :getConnectorAttributeValue %%g "!filePath!" "!fileName!" "port"
            SET portValue=!attributeValue!
            CALL :getConnectorAttributeValue %%g "!filePath!" "!fileName!" "protocol"
            SET protocolType=!attributeValue!
            IF [!portValue!] == [!DEFAULT_RT_PORT!] (
                IF /I "!protocolType:AJP=!" NEQ "!protocolType!" (
                    SET AJP=yes
                ) ELSE (
                    SET AJP=no
                    CALL :migrateConnectorConfig "!filePath!" "!fileName!" %%g "!portValue!" "!AC_PORT_YAMLPATH!" "!AC_MAXTHREADS_YAMLPATH!" "!AC_EXTRACONFIG_YAMLPATH!" "!AC_SENDREASONPHRASE_YAMLPATH!" "!AC_RELAXEDPATHCHARS_YAMLPATH!" "!AC_RELAXEDQUERYCHARS_YAMLPATH!"
                    CALL :setConnectorAttributeValue "!AC_PORT_YAMLPATH!" "port" "!DEFAULT_ACCESS_PORT!"
                    CALL :migrateExtraConnectors "!filePath!" "!fileName!" "!connectorCount!" "yes"
                )
            )
            ENDLOCAL
        )
    )
    IF !nonAjpCount! EQU 0 (
        Rem Add RT connector details as access connector and mark port as 8040  
        CALL :migrateConnectorPort "!filePath!" "!fileName!" "!connectorCount!" "!DEFAULT_RT_PORT!" "!AC_PORT_YAMLPATH!" "!AC_MAXTHREADS_YAMLPATH!" "!AC_EXTRACONFIG_YAMLPATH!" "!AC_SENDREASONPHRASE_YAMLPATH!" "!AC_RELAXEDPATHCHARS_YAMLPATH!" "!AC_RELAXEDQUERYCHARS_YAMLPATH!"
        CALL :setConnectorAttributeValue "!AC_PORT_YAMLPATH!" "port" "!DEFAULT_ACCESS_PORT!"
        Rem migrateExtraConnectors
        CALL :migrateExtraConnectors "!filePath!" "!fileName!" "!connectorCount!" "yes"
    )
    ENDLOCAL
EXIT /B 0

Rem Find for artifactory connector
:findRtConnector
    SETLOCAL EnableDelayedExpansion
    SET filePath=%~1
    SET fileName=%~2
    SET connectorCount=%~3
    SET nonAjpCount=0

    Rem get the count of non AJP 
    CALL :getCountOfNonAjp "!DEFAULT_ACCESS_PORT!" "!connectorCount!" "!filePath!" "!fileName!"
    IF !nonAjpCount! EQU 1 (
        Rem Add the connector found as RT connector
        FOR /L %%g IN (1,1,!connectorCount!) DO (
            SETLOCAL
            CALL :getConnectorAttributeValue %%g "!filePath!" "!fileName!" "port"
            SET portValue=!attributeValue!
            CALL :getConnectorAttributeValue %%g "!filePath!" "!fileName!" "protocol"
            SET protocolType=!attributeValue!
            IF NOT [!portValue!] == [!DEFAULT_ACCESS_PORT!] (
                IF /I "!protocolType:AJP=!" NEQ "!protocolType!" (
                    SET AJP=yes
                    CALL :migrateExtraConnectors "!filePath!" "!fileName!" "!connectorCount!" "no" %%g
                ) ELSE (
                    SET AJP=no
                    CALL :migrateConnectorConfig "!filePath!" "!fileName!" %%g "!portValue!" "!RT_PORT_YAMLPATH!" "!RT_MAXTHREADS_YAMLPATH!" "!RT_EXTRACONFIG_YAMLPATH!" "!RT_SENDREASONPHRASE_YAMLPATH!" "!RT_RELAXEDPATHCHARS_YAMLPATH!" "!RT_RELAXEDQUERYCHARS_YAMLPATH!"
                )  
            )
            ENDLOCAL
        )
    )
    IF !nonAjpCount! GTR 1 (
        Rem Take access properties into artifactory with 8081
        FOR /L %%g IN (1,1,!connectorCount!) DO (
            SETLOCAL
            CALL :getConnectorAttributeValue %%g "!filePath!" "!fileName!" "port"
            SET portValue=!attributeValue!
            CALL :getConnectorAttributeValue %%g "!filePath!" "!fileName!" "protocol"
            SET protocolType=!attributeValue!
            IF [!portValue!] == [!DEFAULT_ACCESS_PORT!] (
                IF /I "!protocolType:AJP=!" NEQ "!protocolType!" (
                    SET AJP=yes
                ) ELSE (
                    SET AJP=no
                    CALL :migrateConnectorConfig "!filePath!" "!fileName!" %%g "!portValue!" "!RT_PORT_YAMLPATH!" "!RT_MAXTHREADS_YAMLPATH!" "!RT_EXTRACONFIG_YAMLPATH!"  "!RT_SENDREASONPHRASE_YAMLPATH!" "!RT_RELAXEDPATHCHARS_YAMLPATH!" "!RT_RELAXEDQUERYCHARS_YAMLPATH!"
                    CALL :setConnectorAttributeValue "!RT_PORT_YAMLPATH!" "port" "!DEFAULT_RT_PORT!"
                    CALL :migrateExtraConnectors "!filePath!" "!fileName!" "!connectorCount!" "yes"
                )
            )
            ENDLOCAL
        )
    )
    IF !nonAjpCount! EQU 0 (
        Rem Add access connector details as RT connector and mark port as !DEFAULT_RT_PORT!
        CALL :migrateConnectorPort "!filePath!" "!fileName!" "!connectorCount!" "!DEFAULT_ACCESS_PORT!" "!RT_PORT_YAMLPATH!" "!RT_MAXTHREADS_YAMLPATH!" "!RT_EXTRACONFIG_YAMLPATH!" "!RT_SENDREASONPHRASE_YAMLPATH!" "!RT_RELAXEDPATHCHARS_YAMLPATH!" "!RT_RELAXEDQUERYCHARS_YAMLPATH!"
        CALL :setConnectorAttributeValue "!RT_PORT_YAMLPATH!" "port" "!DEFAULT_RT_PORT!"
        Rem migrateExtraConnectors
        CALL :migrateExtraConnectors "!filePath!" "!fileName!" "!connectorCount!" "yes"
    )
    ENDLOCAL
EXIT /B 0

:checkForTlsConnector
    SET filePath=%~1
    SET fileName=%~2
    SET connectorCount=%~3
    FOR /L %%g IN (1,1,!connectorCount!) DO (
        CALL :getConnectorAttributeValue %%g "!filePath!" "!fileName!" "sslProtocol"
        SET sslProtocolValue=!attributeValue!
        CALL :getConnectorAttributeValue %%g "!filePath!" "!fileName!" "sslEnabledProtocols"
        SET sslEnabledProtocolsValue=!attributeValue!
        if [!sslProtocolValue!] == [*TLS*] (
            CALL :bannerImportant "NOTE: Ignoring TLS connector during migration, modify the system yaml to enable TLS. Original server.xml is saved in path [!filePath!\!fileName!]"
            SET TLS_CONNECTOR_EXISTS=yes
            EXIT /B 0
        )
        if [!sslEnabledProtocolsValue!] == [*TLS*] (
            CALL :bannerImportant "NOTE: Ignoring TLS connector during migration, modify the system yaml to enable TLS. Original server.xml is saved in path [!filePath!\!fileName!]"
            SET TLS_CONNECTOR_EXISTS=yes
            EXIT /B 0
        )
    )
EXIT /B 0

Rem server.xml migration operations
:xmlMigrationOperation
    SETLOCAL EnableDelayedExpansion
    SET filePath=%~1
    SET fileName=%~2
    SET connectorCount=%~3
    SET listenerConnectorCount=%~4
    SET RT_DEFAULTPORT_STATUS=no
    SET AC_DEFAULTPORT_STATUS=no
    SET TLS_CONNECTOR_EXISTS=no

    Rem Check for connector with TLS , if found ignore migrating it
    CALL :checkForTlsConnector "!filePath!" "!fileName!" "!connectorCount!"
    if [!TLS_CONNECTOR_EXISTS!] == [yes] (
        EXIT /B 0
    )
    Rem  Migrate RT default port from connectors 
    CALL :migrateConnectorPort "!filePath!" "!fileName!" "!connectorCount!" "!DEFAULT_RT_PORT!" "!RT_PORT_YAMLPATH!" "!RT_MAXTHREADS_YAMLPATH!" "!RT_EXTRACONFIG_YAMLPATH!" "!RT_SENDREASONPHRASE_YAMLPATH!" "!RT_RELAXEDPATHCHARS_YAMLPATH!" "!RT_RELAXEDQUERYCHARS_YAMLPATH!"

    Rem  Migrate to extra if RT default ports are AJP
    CALL :migrateDefaultPortIfAjp "!filePath!" "!fileName!" "!connectorCount!" "!DEFAULT_RT_PORT!"

    Rem  Migrate AC default port from connectors 
    CALL :migrateConnectorPort "!filePath!" "!fileName!" "!connectorCount!" "!DEFAULT_ACCESS_PORT!" "!AC_PORT_YAMLPATH!" "!AC_MAXTHREADS_YAMLPATH!" "!AC_EXTRACONFIG_YAMLPATH!" "!AC_SENDREASONPHRASE_YAMLPATH!"
    
    Rem  Migrate to extra if access default ports are AJP
    CALL :migrateDefaultPortIfAjp "!filePath!" "!fileName!" "!connectorCount!" "!DEFAULT_ACCESS_PORT!"

    IF [!AC_DEFAULTPORT_STATUS!/!RT_DEFAULTPORT_STATUS!] == [yes/yes] (
        Rem RT and AC default port found
        CALL :migrateExtraConnectors "!filePath!" "!fileName!" "!connectorCount!" "yes"
        CALL :logger "Artifactory 8081 and Access 8040 default port are found"
    )
    IF [!AC_DEFAULTPORT_STATUS!/!RT_DEFAULTPORT_STATUS!] == [yes/no] (
        Rem Only AC default port found,find RT connector
        CALL :findRtConnector  "!filePath!" "!fileName!" "!connectorCount!"
        CALL :logger "Found Access default 8040 port"
    ) 
    IF [!AC_DEFAULTPORT_STATUS!/!RT_DEFAULTPORT_STATUS!] == [no/yes] (
        Rem Only RT default port found,find AC connector
        CALL :findAcConnector  "!filePath!" "!fileName!" "!connectorCount!"
        CALL :logger "Found Artifactory default 8081 port"
    )
    IF [!AC_DEFAULTPORT_STATUS!/!RT_DEFAULTPORT_STATUS!] == [no/no] (    
        Rem RT and AC default port not found, find connector
        CALL :findRtAndAcConnector  "!filePath!" "!fileName!" "!connectorCount!"
        CALL :logger "Artifactory 8081 and Access 8040 default port are not found"
    )
    ENDLOCAL
EXIT /B 0

Rem get count of connectors
:getXmlConnectorCount
    SET filePath=%~1
    SET fileName=%~2
    FOR /F "tokens=*" %%g IN ('^"!xmllint! --xpath "count(/Server/Service/Connector)" "!filePath!\!fileName!"^"') DO ( SET /a connectorCount=%%g)
EXIT /B 0

Rem get count of listener connectors
:getTomcatServerListenersCount
    SET filePath=%~1
    SET fileName=%~2
    FOR /F "tokens=*" %%g IN ('^"!xmllint! --xpath "count(/Server/Listener)" "!filePath!\!fileName!"^"') DO ( SET /a listenerConnectorCount=%%g)
EXIT /B 0
}

Rem Migrate server.xml configuration to system.yaml
:migrateXmlFile
    SETLOCAL EnableDelayedExpansion
    CALL :retrieveYamlValue "migration.xmlFiles" "xmlFiles" "Skip"
    SET xmlFiles=!YAML_VALUE!
    IF [!xmlFiles!] == [] (
        EXIT /B 0
    )
    CALL :bannerSubSection "PROCESSING MIGRATION OF XML FILES"
    CALL :retrieveYamlValue "migration.xmlFiles.serverXml.fileName" "fileName" "Warning"
    SET fileName=!YAML_VALUE!
    IF [!fileName!] == [] (
        EXIT /B 0
    )
    CALL :bannerSubSection "Processing Migration of !fileName!"
    CALL :retrieveYamlValue "migration.xmlFiles.serverXml.filePath" "filePath" "Warning"
    SET filePath=!YAML_VALUE!
    IF [!filePath!] == [] (
        EXIT /B 0
    )
    SET sourceFilePath=!NEW_DATA_DIR!\!filePath!
    SET fileLocation=!sourceFilePath!\!fileName!
        IF EXIST !fileLocation! (
      	    CALL :logger "File [!fileName!] found in path [!sourceFilePath!]"
            CALL :getXmlConnectorCount "!sourceFilePath!" "!fileName!"
            IF !connectorCount! EQU 0 (
                CALL :logger "No connectors found in the [!fileLocation!],skipping migration of xml configuration"
                EXIT /B 0
            )
            CALL :getTomcatServerListenersCount "!sourceFilePath!" "!fileName!"
            CALL :xmlMigrationOperation "!sourceFilePath!" "!fileName!" "!connectorCount!" "!listenerConnectorCount!"
        ) ELSE (
            CALL :logger "file [!fileName!] is not found in path [!sourceFilePath!] to migrate"
        )
    ENDLOCAL
EXIT /B 0

Rem Check entry in map is format
:checkMapEntry
    SET mapEntry=%~1
    ECHO !mapEntry!|>nul FINDSTR /R "[*=*]" && SET "entryResult=yes" || SET "entryResult=no"
EXIT /B 0

:: Retrieve LHS
:getLeftHandSide
   SET entry=%~1
   IF not [!entry!] == [] (
       FOR /f "tokens=1 delims==" %%f in ("!entry!") DO (
          SET firstEntry=%%f
       )
   )
EXIT /B 0

Rem Retrieve RHS
:getRightHandSide
   SET entry=%~1
   IF not [!entry!] == [] (
       FOR /f "tokens=2 delims==" %%A in ("!entry!") DO (
          SET secondEntry=%%A
       )
   )
EXIT /B 0

Rem Validate %JF_PRODUCT_HOME%\var\etc directory exist
:checkDataDirExists
     SET dataDir=!JF_PRODUCT_HOME!\var\etc
     IF not EXIST !dataDir! mkdir !dataDir!
EXIT /B 0

Rem get time with defined format
:timeFormat
    FOR /f "tokens=2 delims==" %%a IN ('wmic OS Get localdatetime /value') DO SET "dt=%%a"
	SET "YY=%dt:~2,2%" & SET "YYYY=%dt:~0,4%" & SET "MM=%dt:~4,2%" & SET "DD=%dt:~6,2%"
	SET "HH=%dt:~8,2%" & SET "Min=%dt:~10,2%" & SET "Sec=%dt:~12,2%"
	SET "datestamp=%YYYY%%MM%%DD%" & SET "timestamp=%HH%%Min%%Sec%"
	SET "fullstamp=%YYYY%-%MM%-%DD%T%HH%:%Min%:%Sec%"
EXIT /B 0

Rem Create log directory and set log file name
:setMigrationLogFile
    SET targetLog=!JF_PRODUCT_HOME!\var\log
    IF NOT EXIST !targetLog! mkdir !targetLog!
    SET LOG_FILE_NAME=migration.log
    SET MIGRATION_LOG_FILE=!targetLog!\!LOG_FILE_NAME!
EXIT /B 0

Rem Set YQ binary
:initHelper
    SETLOCAL EnableDelayedExpansion
    SET yqBinaryLocation=!JF_PRODUCT_HOME!\app\third-party\yq\yq.exe
    IF defined JF_PRODUCT_HOME IF EXIST !yqBinaryLocation! (
        SET YQ_PATH=!yqBinaryLocation!
    ) ELSE (
        SET JF_ERROR_MSG=Unable to find yq, please set YQ_PATH to parent directory of yq binary
        GOTO :logError !JF_ERROR_MSG!
    )
    ENDLOCAL
EXIT /B 0

:bannerImportant
    SETLOCAL EnableDelayedExpansion
    SET title=%~1
    ECHO.
    ECHO. >> !MIGRATION_LOG_FILE!
    ECHO *************************** IMPORTANT *******************************
    ECHO !title!
    ECHO *********************************************************************
    ECHO *************************** IMPORTANT ******************************* >> !MIGRATION_LOG_FILE! 
    ECHO !title! >> !MIGRATION_LOG_FILE!
    ECHO ********************************************************************* >> !MIGRATION_LOG_FILE!
    ECHO.
    ECHO. >> !MIGRATION_LOG_FILE!
    ENDLOCAL
EXIT /B 0 
:bannerSection
    SETLOCAL EnableDelayedExpansion
    SET title=%~1
    ECHO.
    ECHO. >> !MIGRATION_LOG_FILE!
    ECHO *************************** !title! *******************************
    ECHO *************************** !title! ******************************* >> !MIGRATION_LOG_FILE!
    ECHO.
    ECHO. >> !MIGRATION_LOG_FILE!
    ENDLOCAL
EXIT /B 0

:bannerSubSection
    SETLOCAL EnableDelayedExpansion
    SET title=%~1
    ECHO.
    ECHO. >> !MIGRATION_LOG_FILE!
    ECHO *************** !title! ******************
    ECHO *************** !title! ****************** >> !MIGRATION_LOG_FILE!
    ECHO.
    ECHO. >> !MIGRATION_LOG_FILE!
    ENDLOCAL
EXIT /B 0

Rem Check migration info yaml file available in the path
:checkMigrationInfoYaml
    SET infoYamlLocation=!JF_PRODUCT_HOME!\app\bin\migrationInfo.yaml
    IF defined JF_PRODUCT_HOME IF EXIST "!infoYamlLocation!" (
        SET MIGRATION_SYSTEM_YAML="!infoYamlLocation!"
    ) ELSE (
        SET JF_ERROR_MSG=File migrationInfo.yaml does not exist in path [!infoYamlLocation!]
        GOTO :logError !JF_ERROR_MSG!
    )
EXIT /B 0

Rem Retrieve Product name from MIGRATION_SYSTEM_YAML_INFO
:getProduct
    CALL :retrieveYamlValue "migration.product" !YAML_VALUE! FAIL "Empty value under [migration.product] in [!MIGRATION_SYSTEM_YAML!]" || goto :EOF
    SET PRODUCT=!YAML_VALUE!
    IF not [!PRODUCT!] == [] (
        CALL :toLowerCase PRODUCT 
    )
EXIT /B 0

Rem Set data path of new file system
:getDataDir
    IF [!JF_PRODUCT_HOME!] == [] (
        SET JF_ERROR_MSG=Environment variable JF_PRODUCT_HOME is not set, this is required to perform migration
        GOTO :logError !JF_ERROR_MSG!
    ) ELSE (
        SET NEW_DATA_DIR=!JF_PRODUCT_HOME!\var
    )
EXIT /B 0

Rem Set location of system.yaml
:setSystemYamlPath
   SET SYSTEM_YAML_PATH=!JF_PRODUCT_HOME!\var\etc\system.yaml
   CALL :logger "system.yaml will be created in path [!SYSTEM_YAML_PATH!]"
EXIT /B 0

Rem Subroutine to convert a variable VALUE to all lower case.
Rem The argument for this subroutine is the variable NAME.
:toLowerCase
    FOR %%i IN ("A=a" "B=b" "C=c" "D=d" "E=e" "F=f" "G=g" "H=h" "I=i" "J=j" "K=k" "L=l" "M=m" "N=n" "O=o" "P=p" "Q=q" "R=r" "S=s" "T=t" "U=u" "V=v" "W=w" "X=x" "Y=y" "Z=z") DO CALL SET "%1=%%%1:%%~i%%"
    GOTO :EOF

:logger
    SETLOCAL EnableDelayedExpansion
    CALL :timeFormat || GOTO :EOF
    SET time=!fullstamp!
    ECHO !time! [INFO ] [!scriptName!  ] [main]- %~1
    ECHO !time! [INFO ] [!scriptName!  ] [main]- %~1  >> !MIGRATION_LOG_FILE!
    ENDLOCAL
EXIT /B 0

:logError
    SETLOCAL EnableDelayedExpansion
    CALL :timeFormat || GOTO :EOF
    SET time=!fullstamp!
    ECHO.
    ECHO. >> !MIGRATION_LOG_FILE!
    ECHO !time! [ERROR ] [!scriptName!  ] [main]- !JF_ERROR_MSG!
    ECHO !time! [ERROR ] [!scriptName!  ] [main]- !JF_ERROR_MSG! >> !MIGRATION_LOG_FILE!
    ECHO.
    ECHO. >> !MIGRATION_LOG_FILE!
    ENDLOCAL
EXIT /B 1

:logWarn
    SETLOCAL EnableDelayedExpansion
    CALL :timeFormat || GOTO :EOF
    SET time=!fullstamp!
    ECHO !time! [WARN ] [!scriptName!  ] [main]- !JF_WARN_MSG!
    ECHO !time! [WARN ] [!scriptName!  ] [main]- !JF_WARN_MSG! >> !MIGRATION_LOG_FILE!
    ENDLOCAL
EXIT /B 0

:EOF
ENDLOCAL