package org.artifactory.storage.db.init;

import org.artifactory.common.config.db.ArtifactoryDbProperties;
import org.jfrog.storage.DbProperties;
import org.jfrog.storage.HikariJDBCDataSource;

import javax.annotation.Nonnull;
import javax.sql.DataSource;

import static org.artifactory.common.config.db.ArtifactoryDbProperties.DEFAULT_POOL_TYPE;
import static org.jfrog.storage.DbProperties.TOMCAT_POOL_TYPE;

/**
 * Creates a DataSource from a given {@link ArtifactoryDbProperties}
 *
 * @author Uriah Levy
 */
public class CommonDataSourceFactory {

    private CommonDataSourceFactory() {
    }

    @Nonnull
    public static DataSource createDataSource(ArtifactoryDbProperties dbProps, boolean temporaryDbChannel) {
        String poolType = dbProps.getProperty(DbProperties.Key.POOL_TYPE, DEFAULT_POOL_TYPE);
        return poolType.equalsIgnoreCase(TOMCAT_POOL_TYPE) ?
                new ArtifactoryTomcatDataSource(dbProps, temporaryDbChannel) :
                new ArtifactoryHikariDataSource(dbProps, temporaryDbChannel);
    }

    public static DataSource createUniqueDataSourceByType(ArtifactoryDbProperties dbProps) {
        String poolType = dbProps.getProperty(DbProperties.Key.POOL_TYPE, DEFAULT_POOL_TYPE);
        return poolType.equalsIgnoreCase(TOMCAT_POOL_TYPE) ?
                ArtifactoryTomcatDataSource.createUniqueIdDataSource(dbProps) :
                HikariJDBCDataSource.createUniqueIdDataSource(dbProps);
    }

    public static DataSource createDifferentDatasourceForDbLock(ArtifactoryDbProperties dbProps,
            boolean temporaryChannel) {
        return dbProps.getLockingDbSpecificType().isPresent() ?
                createDatasourceForLockingOnExternalDb(dbProps, temporaryChannel) :
                createDataSource(dbProps, temporaryChannel);
    }

    private static DataSource createDatasourceForLockingOnExternalDb(ArtifactoryDbProperties dbProps,
            boolean temporaryDbChannel) {
        String poolType = dbProps.getProperty(DbProperties.Key.POOL_TYPE, DEFAULT_POOL_TYPE);
        return poolType.equalsIgnoreCase(TOMCAT_POOL_TYPE) ?
                ArtifactoryTomcatDataSource.createDatasourceForLockingOnExternalDb(dbProps, temporaryDbChannel) :
                ArtifactoryHikariDataSource.createDatasourceForLockingOnExternalDb(dbProps, temporaryDbChannel);
    }
}
