/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2019 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.storage.binstore.service;

import com.google.common.collect.ImmutableList;
import org.artifactory.common.ConstantValues;
import org.artifactory.common.home.ArtifactoryHome;
import org.artifactory.common.property.ArtifactorySystemProperties;
import org.artifactory.storage.GCCandidate;
import org.artifactory.test.ArtifactoryHomeBoundTest;
import org.testng.annotations.Test;

import static org.mockito.Mockito.*;
import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertTrue;

/**
 * @author dudim
 */
@Test
public class GCProviderWrapperTest extends ArtifactoryHomeBoundTest {

    public void testErrorCount() {
        ArtifactorySystemProperties artifactoryProperties = ArtifactoryHome.get().getArtifactoryProperties();
        artifactoryProperties.setProperty(ConstantValues.gcFailCountThreshold.getPropertyName(), "3");
        GCProvider gcProviderMock = mock(GCProvider.class);

        doNothing().when(gcProviderMock).resetProvider();
        when(gcProviderMock.getBatch()).thenReturn(ImmutableList.of(
                new GCCandidate(null, "isEnabled=true", null, null, 0)));

        when(gcProviderMock.getAction()).thenReturn((o, o2) -> {
            throw new RuntimeException("increasing error count");
        });

        GCProviderWrapper gcProviderWrapper = new GCProviderWrapper(gcProviderMock);
        gcProviderWrapper.resetProvider();
        assertEquals(gcProviderWrapper.getBatch().get(0).getSha1(), "isEnabled=true");

        gcProviderWrapper.getAction()
                .accept(new GCCandidate(null, null, null, null, 0), new GarbageCollectorInfo());
        assertEquals(gcProviderWrapper.getBatch().get(0).getSha1(), "isEnabled=true");

        gcProviderWrapper.getAction()
                .accept(new GCCandidate(null, null, null, null, 0), new GarbageCollectorInfo());
        assertEquals(gcProviderWrapper.getBatch().get(0).getSha1(), "isEnabled=true");

        gcProviderWrapper.getAction()
                .accept(new GCCandidate(null, null, null, null, 0), new GarbageCollectorInfo());
        assertTrue(gcProviderWrapper.getBatch().isEmpty());
    }
}