package org.artifactory.storage.db.init;

import org.artifactory.common.config.db.ArtifactoryDbProperties;
import org.artifactory.test.ArtifactoryHomeBoundTest;
import org.jfrog.storage.DbType;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import static org.artifactory.common.config.db.ArtifactoryDbProperties.DEFAULT_POOL_TYPE;
import static org.mockito.Mockito.when;
import static org.testng.Assert.assertEquals;

@Test
public class CommonDataSourceFactoryTest extends ArtifactoryHomeBoundTest {

    @Mock
    ArtifactoryDbProperties artifactoryDbProperties;

    @BeforeMethod
    public void setUp() {
        MockitoAnnotations.initMocks(this);
    }

    @Test
    public void artifactoryTestInitTomcatDataSourceParameters() {
        when(artifactoryDbProperties.getProperty(ArtifactoryDbProperties.Key.POOL_TYPE, DEFAULT_POOL_TYPE))
                .thenReturn("tomcat-jdbc");
        when(artifactoryDbProperties.getMaxIdleConnections()).thenReturn(5);
        when(artifactoryDbProperties.getIntProperty("minIdle", 1)).thenReturn(5);
        when(artifactoryDbProperties.getMaxActiveConnections()).thenReturn(5);
        when(artifactoryDbProperties.getDbType()).thenReturn(DbType.DERBY);

        ArtifactoryTomcatDataSource artifactoryTomcatDataSource = (ArtifactoryTomcatDataSource) CommonDataSourceFactory
                .createDataSource(artifactoryDbProperties, true);

        assertEquals(artifactoryTomcatDataSource.getMaxActive(), 1);
        assertEquals(artifactoryTomcatDataSource.getMaxIdle(), 1);
        assertEquals(artifactoryTomcatDataSource.getMinIdle(), 0);
    }

    @Test
    public void artifactoryTestTomcatDataSourceParameters() {
        when(artifactoryDbProperties.getProperty(ArtifactoryDbProperties.Key.POOL_TYPE, DEFAULT_POOL_TYPE))
                .thenReturn("tomcat-jdbc");
        when(artifactoryDbProperties.getMaxIdleConnections()).thenReturn(5);
        when(artifactoryDbProperties.getMaxActiveConnections()).thenReturn(5);
        when(artifactoryDbProperties.getIntProperty("minIdle", 1)).thenReturn(5);
        when(artifactoryDbProperties.getDbType()).thenReturn(DbType.DERBY);

        ArtifactoryTomcatDataSource artifactoryTomcatDataSource = (ArtifactoryTomcatDataSource) CommonDataSourceFactory
                .createDataSource(artifactoryDbProperties, false);

        assertEquals(artifactoryTomcatDataSource.getMaxActive(), 5);
        assertEquals(artifactoryTomcatDataSource.getMaxIdle(), 5);
        assertEquals(artifactoryTomcatDataSource.getMinIdle(), 5);
    }

    @Test
    public void artifactoryTestInitHikariDataSourceParameters() {
        when(artifactoryDbProperties.getProperty(ArtifactoryDbProperties.Key.POOL_TYPE, DEFAULT_POOL_TYPE))
                .thenReturn("hikari");
        when(artifactoryDbProperties.getMaxActiveConnections()).thenReturn(5);
        when(artifactoryDbProperties.getDbType()).thenReturn(DbType.DERBY);
        when(artifactoryDbProperties.getDriverClass()).thenReturn("org.apache.derby.jdbc.EmbeddedDriver");
        when(artifactoryDbProperties.getIntProperty("validationQueryTimeout", 5000)).thenReturn(5000);

        ArtifactoryHikariDataSource artifactoryHikariDataSource = (ArtifactoryHikariDataSource) CommonDataSourceFactory
                .createDataSource(artifactoryDbProperties, true);

        assertEquals(artifactoryHikariDataSource.getMaximumPoolSize(), 1);
        assertEquals(artifactoryHikariDataSource.getMinimumIdle(), 0);
    }

    @Test
    public void artifactoryTestHikariDataSourceParameters() {
        when(artifactoryDbProperties.getProperty(ArtifactoryDbProperties.Key.POOL_TYPE, DEFAULT_POOL_TYPE))
                .thenReturn("hikari");
        when(artifactoryDbProperties.getMaxActiveConnections()).thenReturn(5);
        when(artifactoryDbProperties.getDbType()).thenReturn(DbType.DERBY);
        when(artifactoryDbProperties.getIntProperty("minIdle", 1)).thenReturn(1);
        when(artifactoryDbProperties.getDriverClass()).thenReturn("org.apache.derby.jdbc.EmbeddedDriver");
        when(artifactoryDbProperties.getIntProperty("validationQueryTimeout", 5000)).thenReturn(5000);

        ArtifactoryHikariDataSource artifactoryHikariDataSource = (ArtifactoryHikariDataSource) CommonDataSourceFactory
                .createDataSource(artifactoryDbProperties, false);

        assertEquals(artifactoryHikariDataSource.getMaximumPoolSize(), 5);
        assertEquals(artifactoryHikariDataSource.getMinimumIdle(), 1);
    }
}