package org.artifactory.storage.db.conversion.version.v220;

import lombok.NonNull;
import org.artifactory.storage.db.version.ConversionPredicate;
import org.jfrog.storage.DbType;
import org.jfrog.storage.JdbcHelper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.function.BiPredicate;
import static org.jfrog.storage.util.DbUtils.getColumnSize;

/**
 * @author Shay Bagants
 */
public class V220ConversionPredicate implements ConversionPredicate {
    private static final Logger log = LoggerFactory.getLogger(V220ConversionPredicate.class);

    @NonNull
    @Override
    public BiPredicate<JdbcHelper, DbType> condition() {
        return (jdbcHelper, dbType) -> {
            try {
                // run conversion if column task_context is not equals to 2048
                return getColumnSize(jdbcHelper, dbType, "tasks", "task_context") != 2048;
            } catch (Exception e) {
                log.error("Cannot run conversion 'v220' - Failed to retrieve schema metadata: ", e);
            }
            return false;
        };
    }
}
