package org.artifactory.storage.db.binstore.service;

import org.artifactory.addon.AddonsManager;
import org.artifactory.addon.CoreAddons;
import org.artifactory.api.context.ArtifactoryContext;
import org.artifactory.api.context.ArtifactoryContextThreadBinder;
import org.artifactory.api.context.ContextHelper;
import org.artifactory.common.home.ArtifactoryHome;
import org.jfrog.storage.binstore.config.model.ProviderMetaData;
import org.jfrog.storage.binstore.ifc.BinaryProviderManager;
import org.jfrog.storage.binstore.providers.base.CloudBinaryProvider;
import org.jfrog.storage.binstore.providers.base.MutableBinaryProvider;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import static org.mockito.Mockito.when;
import static org.testng.Assert.*;

/**
 * Author: Ivan Perepelytsya
 * Date: 6/3/20.
 */
public class BinaryServiceImplTest {

    @InjectMocks
    private BinaryServiceImpl binaryService;

    @Mock
    private BinaryProviderManager binaryProviderManager;
    @Mock
    private CloudBinaryProvider cloudBinaryProvider;
    @Mock
    private AddonsManager addonsManager;
    @Mock
    private CoreAddons coreAddons;
    @Mock
    private MutableBinaryProvider mutableBinaryProvider;
    @Mock
    private ProviderMetaData providerMetaData;
    @Mock
    private ArtifactoryContext artifactoryContext;
    @Mock
    private ArtifactoryHome artifactoryHome;

    @BeforeMethod
    public void initMocks(){
        MockitoAnnotations.initMocks(this);

        ArtifactoryContextThreadBinder.bind(artifactoryContext);

        ArtifactoryHome.bind(artifactoryHome);
        when(ContextHelper.get().beanForType(AddonsManager.class)).thenReturn(addonsManager);
        when(binaryProviderManager.getNextCloudProvider()).thenReturn(cloudBinaryProvider);
        when(cloudBinaryProvider.getMutableBinaryProvider()).thenReturn(mutableBinaryProvider);
        when(mutableBinaryProvider.getProviderMetaData()).thenReturn(providerMetaData);
        when(addonsManager.addonByType(CoreAddons.class)).thenReturn(coreAddons);
    }

    @AfterMethod
    private void cleanup() {
        ArtifactoryContextThreadBinder.unbind();
        ArtifactoryHome.unbind();
    }


    @Test
    public void testCdnIsAllowed() {
        when(coreAddons.isAol()).thenReturn(true);
        when(providerMetaData.getType()).thenReturn("s3-storage-v3");
        when(cloudBinaryProvider.supportsRedirection()).thenReturn(true);
        when(addonsManager.isEnterpriseLicensed()).thenReturn(true);
        final boolean cdnAllowed = binaryService.isCdnAllowed();
        assertTrue(cdnAllowed);
    }

    @Test
    public void testCdnIsNotAllowedNotAol() {
        when(coreAddons.isAol()).thenReturn(false);
        when(providerMetaData.getType()).thenReturn("s3-storage-v3");
        when(cloudBinaryProvider.supportsRedirection()).thenReturn(true);
        when(addonsManager.isEnterpriseLicensed()).thenReturn(true);
        final boolean cdnAllowed = binaryService.isCdnAllowed();
        assertFalse(cdnAllowed);
    }

    @Test
    public void testCdnIsNotAllowedNotS3() {
        when(providerMetaData.getType()).thenReturn("GCP");
        when(cloudBinaryProvider.supportsRedirection()).thenReturn(true);
        when(addonsManager.isEnterpriseLicensed()).thenReturn(true);
        final boolean cdnAllowed = binaryService.isCdnAllowed();
        assertFalse(cdnAllowed);
    }

    @Test
    public void testCdnIsNotAllowedNoRedirects() {
        when(providerMetaData.getType()).thenReturn("s3-storage-v3");
        when(cloudBinaryProvider.supportsRedirection()).thenReturn(false);
        when(addonsManager.isEnterpriseLicensed()).thenReturn(true);
        final boolean cdnAllowed = binaryService.isCdnAllowed();
        assertFalse(cdnAllowed);
    }

    @Test
    public void testCdnIsNotAllowedWrongLicense() {
        when(providerMetaData.getType()).thenReturn("s3-storage-v3");
        when(cloudBinaryProvider.supportsRedirection()).thenReturn(true);
        final boolean cdnAllowed = binaryService.isCdnAllowed();
        assertFalse(cdnAllowed);
    }

}