/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2018 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.storage.db.ds.itest;

import com.jfrog.sysconf.SysLayout;
import org.artifactory.common.config.db.ArtifactoryDbProperties;
import org.artifactory.common.home.ArtifactoryHome;
import org.artifactory.storage.db.init.ArtifactoryDataSource;
import org.artifactory.storage.db.init.ArtifactoryHikariDataSource;
import org.artifactory.storage.db.itest.DbBaseTest;
import org.artifactory.test.TestUtils;
import org.artifactory.util.Files;
import org.jfrog.common.ResourceUtils;
import org.jfrog.storage.HikariJDBCDataSource;
import org.springframework.beans.factory.annotation.Autowired;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import javax.sql.DataSource;
import java.io.File;
import java.io.IOException;
import java.sql.Connection;
import java.sql.SQLException;

import static org.jfrog.config.home.JFrogHome.SYSTEM_YAML_FILENAME;
import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertNotNull;
import static org.testng.Assert.assertTrue;

/**
 * Tests the connection pooling done with {@link ArtifactoryHikariDataSource}.
 *
 * @author Yossi Shaul
 */
@Test
public class ArtifactoryHikariDataSourceTest extends DbBaseTest {

    @Autowired
    private DataSource dataSource;

    private boolean isHikariDS;

    @BeforeClass
    public void setup() {
        isHikariDS = dataSource instanceof ArtifactoryHikariDataSource;
    }

    public void instanceType() {
        assertTrue(dataSource instanceof ArtifactoryDataSource, "Unexpected datasource type: " + dataSource.getClass());
    }

    public void verifyDefaultParams() {
        if (isHikariDS) {
            ArtifactoryHikariDataSource ds = (ArtifactoryHikariDataSource) dataSource;
            assertEquals(ds.getTransactionIsolation(), "TRANSACTION_READ_COMMITTED");
            assertTrue(ds.isAutoCommit());
        }
    }

    public void verifyActiveConnections() throws SQLException {
        assertEquals(getDs().getActiveConnectionsCount(), 0);
        //assertEquals(getDs().getIdleConnectionsCount(), dbProps.get);
        try (Connection conn = getDs().getConnection()) {
            assertNotNull(conn);
            assertEquals(getDs().getActiveConnectionsCount(), 1);
        }
        assertEquals(getDs().getActiveConnectionsCount(), 0);
    }

    public void lockingDbDataSource() throws IOException {
        File home = TestUtils.createTempDir(getClass());
        SysLayout sysLayout = new SysLayout(home, "test");
        assertTrue(sysLayout.getProductEtc().mkdirs());
        Files.writeToFileAndClose(ResourceUtils.getResource("/db/lockingdb.yaml"),
                new File(sysLayout.getProductEtc() + "/" + SYSTEM_YAML_FILENAME));
        var dbProps = new ArtifactoryDbProperties(new ArtifactoryHome(home.getAbsolutePath()));
        DataSource ds = ArtifactoryHikariDataSource.createDatasourceForLockingOnExternalDb(dbProps, false);
        assertEquals(ds.getClass(), HikariJDBCDataSource.class);

        HikariJDBCDataSource hds = (HikariJDBCDataSource) ds;
        // check non locking db parameters
        assertEquals(hds.getMaximumPoolSize(), 15);

        // check locking db overrides
        assertEquals(hds.getDriverClassName(), "org.postgresql.Driver");
        assertEquals(hds.getJdbcUrl(), "jdbc:postgresql://localhost:5432/lockdb");
        assertEquals(hds.getUsername(), "lockdbuser");
        assertEquals(hds.getPassword(), "lockdbpass");
        assertEquals(hds.getCatalog(), "lockingcatalog");
    }

    private ArtifactoryDataSource getDs() {
        return (ArtifactoryDataSource) dataSource;
    }
}
