package org.artifactory.storage.db.upgrades.itest.version;

import org.artifactory.storage.db.conversion.version.v220.V220ConversionPredicate;
import org.artifactory.storage.db.upgrades.common.UpgradeBaseTest;
import org.artifactory.storage.db.version.ArtifactoryDBVersion;
import org.artifactory.storage.db.version.converter.DBConverter;
import org.jfrog.storage.DbType;
import org.jfrog.storage.JdbcHelper;
import org.testng.annotations.Test;

import java.io.IOException;
import java.sql.SQLException;
import java.util.function.BiPredicate;

import static org.jfrog.storage.util.DbUtils.withConnection;
import static org.testng.Assert.*;

/**
 * @author Shay Bagants
 */
@Test
public class V220ChangeTasksContextSizeTest extends UpgradeBaseTest {

    @Test
    public void testConvertColumn() throws IOException, SQLException {
        resetToVersion(ArtifactoryDBVersion.v219);
        String columnName = "task_context";
        String tableName = "tasks";
        assertTrue(columnExists(tableName, columnName));
        int columnSize = withConnection(jdbcHelper, conn -> columnSize(tableName, columnName));
        assertEquals(columnSize, 1024);
        BiPredicate<JdbcHelper, DbType> condition = new V220ConversionPredicate().condition();
        boolean shouldConvert = condition.test(jdbcHelper, dbProperties.getDbType());
        assertTrue(shouldConvert);

        for (DBConverter dbConverter : ArtifactoryDBVersion.v220.getConverters()) {
            dbConverter.convert(jdbcHelper, dbProperties.getDbType());
        }

        shouldConvert = condition.test(jdbcHelper, dbProperties.getDbType());
        assertFalse(shouldConvert);
        columnSize = withConnection(jdbcHelper, conn -> columnSize(tableName, columnName));
        assertEquals(columnSize, 2048);
    }
}
