package org.artifactory.webapp.servlet;

import com.google.common.collect.Iterators;
import org.artifactory.api.context.ArtifactoryContext;
import org.artifactory.opentracing.ArtifactoryTracer;
import org.artifactory.opentracing.ServerTracingScope;

import javax.servlet.*;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.Map;
import java.util.stream.Collectors;
import java.util.stream.StreamSupport;

/**
 * @author Uriah Levy
 */
public class ArtifactoryTracingFilter extends DelayedFilterBase {

    private ArtifactoryTracer artifactoryTracer;

    @Override
    public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain)
            throws IOException, ServletException {
        try (ServerTracingScope ignored = startServerSpan((HttpServletRequest) request,
                ((HttpServletRequest) request).getPathInfo())) {
            chain.doFilter(request, response);
        }
    }

    @Override
    public void destroy() {
        if (artifactoryTracer != null) {
            artifactoryTracer.closeTracer();
        }
    }

    @Override
    protected void initLater(FilterConfig config) {
        ServletContext servletContext = config.getServletContext();
        ArtifactoryContext context = RequestUtils.getArtifactoryContext(servletContext);
        this.artifactoryTracer = context.beanForType(ArtifactoryTracer.class);
    }

    private ServerTracingScope startServerSpan(HttpServletRequest request, String requestPath) {
        Map<String, String> headers = StreamSupport
                .stream(((Iterable<String>) () -> Iterators.forEnumeration(request.getHeaderNames())).spliterator(),
                        false)
                .collect(Collectors.toMap(k -> k, request::getHeader));
        return artifactoryTracer.startSpanOnCurrentThread(requestPath, headers);
    }

    void setArtifactoryTracer(ArtifactoryTracer tracer) {
        this.artifactoryTracer = tracer;
    }
}
