package org.artifactory.ui.rest.model.admin.configuration.repository.info;

import org.artifactory.addon.AddonsManager;
import org.artifactory.addon.webstart.ArtifactWebstartAddon;
import org.artifactory.api.config.CentralConfigService;
import org.artifactory.api.context.ArtifactoryContextThreadBinder;
import org.artifactory.api.context.ContextHelper;
import org.artifactory.api.repo.RepositoryService;
import org.artifactory.common.home.ArtifactoryHome;
import org.artifactory.common.property.ArtifactorySystemProperties;
import org.artifactory.descriptor.config.CentralConfigDescriptor;
import org.artifactory.spring.ArtifactoryApplicationContext;
import org.artifactory.storage.binstore.service.BinaryService;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import java.util.List;

import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;
import static org.testng.Assert.*;

/**
 * Author: Ivan Perepelytsya
 * Date: 6/3/20.
 */
@Test
public class RepositoryFieldsValuesModelTest {

    @Mock
    private ArtifactoryApplicationContext artifactoryContext;
    @Mock
    private AddonsManager addonsManager;
    @Mock
    private ArtifactWebstartAddon artifactWebstartAddon;
    @Mock
    private CentralConfigDescriptor descriptor;
    @Mock
    private CentralConfigService centralConfigService;
    @Mock
    private ArtifactoryHome artifactoryHome;
    @Mock
    private ArtifactorySystemProperties artifactorySystemProperties;

    @Test
    public void testCdn() {
        final RepositoryService repositoryService = mock(RepositoryService.class);
        final BinaryService binaryService = mock(BinaryService.class);

        when(binaryService.isCdnAllowed()).thenReturn(true);
        when(descriptor.getProxies()).thenReturn(List.of());
        final RepositoryFieldsValuesModel model =
                new RepositoryFieldsValuesModel(descriptor, repositoryService, binaryService, List.of("mvn","npm", "nuget"));
        final List<String> allowedRedirects = model.getCdnRedirectAllowedRepos();

        assertEquals(allowedRedirects.size(), 3);
        assertEquals(allowedRedirects.get(0), "mvn");
        assertEquals(allowedRedirects.get(1), "npm");
        assertEquals(allowedRedirects.get(2), "nuget");
        assertTrue(model.isCdnAllowedPerRepo());

    }

    @BeforeMethod
    public void setup() {
        MockitoAnnotations.initMocks(this);
        ArtifactoryContextThreadBinder.bind(artifactoryContext);

        ArtifactoryHome.bind(artifactoryHome);
        when(artifactoryHome.getArtifactoryProperties()).thenReturn(artifactorySystemProperties);

        when(addonsManager.addonByType(ArtifactWebstartAddon.class)).thenReturn(artifactWebstartAddon);
        when(artifactWebstartAddon.getKeyPairNames()).thenReturn(List.of());
        when(ContextHelper.get().beanForType(AddonsManager.class)).thenReturn(addonsManager);
        when(ContextHelper.get().beanForType(CentralConfigService.class)).thenReturn(centralConfigService);
        when(centralConfigService.getDescriptor()).thenReturn(descriptor);
    }

    @AfterMethod
    private void cleanup() {
        ArtifactoryContextThreadBinder.unbind();
        ArtifactoryHome.unbind();
    }
}