package org.artifactory.ui.rest.service.admin.configuration.repositories.util.builder;

import org.artifactory.addon.AddonsManager;
import org.artifactory.addon.CoreAddons;
import org.artifactory.api.config.CentralConfigService;
import org.artifactory.api.context.ArtifactoryContext;
import org.artifactory.api.context.ArtifactoryContextThreadBinder;
import org.artifactory.common.ConstantValues;
import org.artifactory.common.home.ArtifactoryHome;
import org.artifactory.common.property.ArtifactorySystemProperties;
import org.artifactory.descriptor.config.MutableCentralConfigDescriptor;
import org.artifactory.descriptor.repo.LocalRepoDescriptor;
import org.artifactory.descriptor.repo.RemoteRepoDescriptor;
import org.artifactory.descriptor.repo.RepoType;
import org.artifactory.engine.DownloadRedirectHelper;
import org.artifactory.storage.binstore.service.BinaryService;
import org.artifactory.ui.rest.model.admin.configuration.repository.BasicRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.CdnRedirectRepoConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.GeneralRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.local.LocalAdvancedRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.local.LocalBasicRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.local.LocalRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.remote.RemoteAdvancedRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.remote.RemoteBasicRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.remote.RemoteRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.NpmTypeSpecificConfigModel;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.when;
import static org.testng.Assert.*;

/**
 * Author: Ivan Perepelytsya
 * Date: 6/3/20.
 */
@Test
public class RepoConfigDescriptorBuilderTest {

    @Mock
    private DownloadRedirectHelper downloadRedirectHelper;
    @Mock
    private AddonsManager addonsManager;
    @Mock
    private BinaryService binaryService;
    @Mock
    private CoreAddons coreAddons;
    @Mock
    private NpmTypeSpecificConfigModel typeSpecificConfigModel;
    @Mock
    private ArtifactoryContext artifactoryContext;
    @Mock
    private ArtifactoryHome artifactoryHome;
    @Mock
    private CentralConfigService centralConfigService;
    @Mock
    private MutableCentralConfigDescriptor mutableCentralConfigDescriptor;
    @Mock
    private ArtifactorySystemProperties artifactorySystemProperties;
    @InjectMocks
    private RepoConfigDescriptorBuilder repoConfigDescriptorBuilder;


    @Test
    public void testBuildLocalDescriptor() {
        final LocalRepositoryConfigModel model = prepareLocalModel();
        final LocalRepoDescriptor localRepoDescriptor = repoConfigDescriptorBuilder.buildLocalDescriptor(model);
        assertTrue(localRepoDescriptor.getCdnRedirectRepoConfig().isEnabled());
    }

    @Test
    public void testBuildRemoteDescriptor() {
        final RemoteRepositoryConfigModel model = prepareRemoteModel();
        final RemoteRepoDescriptor localRepoDescriptor = repoConfigDescriptorBuilder.buildRemoteDescriptor(model);
        assertTrue(localRepoDescriptor.getCdnRedirectRepoConfig().isEnabled());
    }

    private void fillBasic(BasicRepositoryConfigModel basic){
        basic.setPublicDescription("");
        basic.setInternalDescription("");
        basic.setIncludesPattern("");
        basic.setExcludesPattern("");
    }

    private LocalRepositoryConfigModel prepareLocalModel(){

        final LocalRepositoryConfigModel model = new LocalRepositoryConfigModel();
        CdnRedirectRepoConfigModel cdnRedirectRepoConfig = new CdnRedirectRepoConfigModel();
        cdnRedirectRepoConfig.setEnabled(true);
        final LocalAdvancedRepositoryConfigModel advanced = new LocalAdvancedRepositoryConfigModel();
        final GeneralRepositoryConfigModel general = new GeneralRepositoryConfigModel();
        final LocalBasicRepositoryConfigModel basic = new LocalBasicRepositoryConfigModel();
        fillBasic(basic);
        general.setRepoKey("repo");

        advanced.setCdnRedirectRepoConfig(cdnRedirectRepoConfig);

        model.setTypeSpecific(typeSpecificConfigModel);
        model.setAdvanced(advanced);
        model.setGeneral(general);
        model.setBasic(basic);

        return model;
    }

    private RemoteRepositoryConfigModel prepareRemoteModel(){

        final RemoteRepositoryConfigModel model = new RemoteRepositoryConfigModel();
        CdnRedirectRepoConfigModel cdnRedirectRepoConfig = new CdnRedirectRepoConfigModel();
        cdnRedirectRepoConfig.setEnabled(true);
        final RemoteAdvancedRepositoryConfigModel advanced = new RemoteAdvancedRepositoryConfigModel();
        final GeneralRepositoryConfigModel general = new GeneralRepositoryConfigModel();
        final RemoteBasicRepositoryConfigModel basic = new RemoteBasicRepositoryConfigModel();
        fillBasic(basic);

        general.setRepoKey("repo");

        advanced.setCdnRedirectRepoConfig(cdnRedirectRepoConfig);

        model.setTypeSpecific(typeSpecificConfigModel);
        model.setAdvanced(advanced);
        model.setGeneral(general);
        model.setBasic(basic);
        when(artifactorySystemProperties.getProperty(any(ConstantValues.class)))
                .thenAnswer(invocationOnMock -> {
                    ConstantValues constantValue = invocationOnMock.getArgument(0);
                    if (constantValue == ConstantValues.debianDefaultArchitectures) {
                        return "deb";
                    }
                    return constantValue.getDefValue().trim();
                });

        return model;
    }


    @BeforeMethod
    public void initMocks(){
        MockitoAnnotations.initMocks(this);
        ArtifactoryContextThreadBinder.bind(artifactoryContext);
        ArtifactoryHome.bind(artifactoryHome);
        when(artifactoryContext.getCentralConfig()).thenReturn(centralConfigService);
        when(centralConfigService.getMutableDescriptor()).thenReturn(mutableCentralConfigDescriptor);
        when(mutableCentralConfigDescriptor.getCurrentReverseProxy()).thenReturn(null);
        when(artifactoryHome.getArtifactoryProperties()).thenReturn(artifactorySystemProperties);

        when(downloadRedirectHelper.isRedirectEnableForThisRepoType(anyString())).thenReturn(true);
        when(binaryService.isCdnAllowed()).thenReturn(true);
        when(addonsManager.addonByType(CoreAddons.class)).thenReturn(coreAddons);
        when(coreAddons.isAol()).thenReturn(true);
        when(typeSpecificConfigModel.getRepoType()).thenReturn(RepoType.Npm);

    }

    @AfterMethod
    private void cleanup() {
        ArtifactoryContextThreadBinder.unbind();
        ArtifactoryHome.unbind();
    }
}