package org.artifactory.ui.rest.service.admin.configuration.repositories.util.builder;

import org.artifactory.addon.AddonsManager;
import org.artifactory.addon.webstart.ArtifactWebstartAddon;
import org.artifactory.api.config.CentralConfigService;
import org.artifactory.common.home.ArtifactoryHome;
import org.artifactory.common.property.ArtifactorySystemProperties;
import org.artifactory.descriptor.Descriptor;
import org.artifactory.descriptor.config.CentralConfigDescriptor;
import org.artifactory.descriptor.repo.CdnRedirectRepoConfig;
import org.artifactory.descriptor.repo.HttpRepoDescriptor;
import org.artifactory.descriptor.repo.LocalRepoDescriptor;
import org.artifactory.descriptor.repo.RemoteRepoDescriptor;
import org.artifactory.descriptor.repo.ReverseProxyDescriptor;
import org.artifactory.ui.rest.model.admin.configuration.repository.local.LocalRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.remote.RemoteRepositoryConfigModel;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import java.util.List;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;
import static org.testng.Assert.*;

/**
 * Author: Ivan Perepelytsya
 * Date: 6/2/20.
 */
@Test
public class RepoConfigModelBuilderTest {

    @BeforeMethod
    public void bindArtifactoryHome() {
        final ArtifactorySystemProperties artifactorySystemProperties = mock(ArtifactorySystemProperties.class);
        final ArtifactoryHome artifactoryHome = mock(ArtifactoryHome.class);
        when(artifactoryHome.getArtifactoryProperties()).thenReturn(artifactorySystemProperties);
        when(artifactorySystemProperties.getProperty(any())).thenReturn("");
        ArtifactoryHome.bind(artifactoryHome);
    }


    @AfterMethod
    public void unbindArtifactoryHome() {
        ArtifactoryHome.unbind();
    }

    @Test
    public void testPopulateLocalDescriptorCdn() {
        final RepoConfigModelBuilder repoConfigModelBuilder = new RepoConfigModelBuilder();
        repoConfigModelBuilder.setCentralConfig(getCentralConfigService());

        final LocalRepoDescriptor localRepoDescriptor = new LocalRepoDescriptor();
        localRepoDescriptor.setCdnRedirectConfig(getEnabledCdnConfig());

        final LocalRepositoryConfigModel localRepositoryConfigModel = new LocalRepositoryConfigModel();
        repoConfigModelBuilder.populateLocalDescriptorValuesToModel(localRepoDescriptor, localRepositoryConfigModel);

        assertTrue(localRepositoryConfigModel.getAdvanced().getCdnRedirectRepoConfig().isEnabled());
    }

    @Test
    public void testPopulateRemoteDescriptorCdn() {
        final RepoConfigModelBuilder repoConfigModelBuilder = getRepoConfigModelBuilder();

        final HttpRepoDescriptor remoteRepoDescriptor = new HttpRepoDescriptor();
        remoteRepoDescriptor.setCdnRedirectConfig(getEnabledCdnConfig());

        final RemoteRepositoryConfigModel remoteRepositoryConfigModel = new RemoteRepositoryConfigModel();
        repoConfigModelBuilder
                .populateRemoteRepositoryConfigValuesToModel(remoteRepoDescriptor, remoteRepositoryConfigModel);

        assertTrue(remoteRepositoryConfigModel.getAdvanced().getCdnRedirectRepoConfig().isEnabled());
    }

    @Test
    public void testPopulateLocalDescriptorNullCdn() {

        final RepoConfigModelBuilder repoConfigModelBuilder = new RepoConfigModelBuilder();
        repoConfigModelBuilder.setCentralConfig(getCentralConfigService());

        final LocalRepoDescriptor localRepoDescriptor = new LocalRepoDescriptor();
        localRepoDescriptor.setCdnRedirectConfig(null);

        final LocalRepositoryConfigModel localRepositoryConfigModel = new LocalRepositoryConfigModel();
        repoConfigModelBuilder.populateLocalDescriptorValuesToModel(localRepoDescriptor, localRepositoryConfigModel);
        assertNull(localRepositoryConfigModel.getAdvanced().getCdnRedirectRepoConfig());
    }



    @Test
    public void testPopulateRemoteDescriptorNullCdn() {
        final RepoConfigModelBuilder repoConfigModelBuilder = getRepoConfigModelBuilder();

        final HttpRepoDescriptor remoteRepoDescriptor = new HttpRepoDescriptor();
        remoteRepoDescriptor.setCdnRedirectConfig(null);

        final RemoteRepositoryConfigModel remoteRepositoryConfigModel = new RemoteRepositoryConfigModel();
        repoConfigModelBuilder
                .populateRemoteRepositoryConfigValuesToModel(remoteRepoDescriptor, remoteRepositoryConfigModel);

        assertNull(remoteRepositoryConfigModel.getAdvanced().getCdnRedirectRepoConfig());
    }

    private CentralConfigService getCentralConfigService() {
        final CentralConfigService centralConfigService = mock(CentralConfigService.class);
        final CentralConfigDescriptor centralConfigDescriptor = mock(CentralConfigDescriptor.class);

        when(centralConfigService.getDescriptor()).thenReturn(centralConfigDescriptor);
        return centralConfigService;
    }

    private CdnRedirectRepoConfig getEnabledCdnConfig(){
        CdnRedirectRepoConfig cdnRedirectRepoConfig = new CdnRedirectRepoConfig();
        cdnRedirectRepoConfig.setEnabled(true);
        return cdnRedirectRepoConfig;
    }


    private RepoConfigModelBuilder getRepoConfigModelBuilder(){
        final RepoConfigModelBuilder repoConfigModelBuilder = new RepoConfigModelBuilder();
        repoConfigModelBuilder.setCentralConfig(getCentralConfigService());
        final AddonsManager addonsManager = mock(AddonsManager.class);
        final ArtifactWebstartAddon artifactWebstartAddon = mock(ArtifactWebstartAddon.class);
        when(artifactWebstartAddon.getSslCertNames()).thenReturn(List.of());
        when(addonsManager.addonByType(ArtifactWebstartAddon.class)).thenReturn(artifactWebstartAddon);
        repoConfigModelBuilder.setAddonsManager(addonsManager);
        return repoConfigModelBuilder;
    }



}